# Copyright 2024 Cisco Systems, Inc. and its affiliates

from typing import List, Optional, Union

from pydantic import BaseModel, ConfigDict, Field

from catalystwan.api.configuration_groups.parcel import Default, Global, Variable


class OptionCodeAscii(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    code: Union[Global[int], Variable]
    ascii: Union[Global[str], Variable]


class OptionCodeHex(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    code: Union[Global[int], Variable]
    hex: Union[Global[str], Variable]


class OptionCodeIP(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    code: Union[Global[int], Variable]
    ip: Union[Global[List[str]], Variable]


class StaticLease(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    mac_address: Union[Global[str], Variable] = Field(serialization_alias="macAddress", validation_alias="macAddress")
    ip: Union[Global[str], Variable]


class DhcpAddressPool(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    network_address: Union[Global[str], Variable] = Field(
        serialization_alias="networkAddress", validation_alias="networkAddress"
    )
    subnet_mask: Union[Global[str], Variable] = Field(serialization_alias="subnetMask", validation_alias="subnetMask")


class DhcpServerData(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    address_pool: DhcpAddressPool = Field(serialization_alias="addressPool", validation_alias="addressPool")
    exclude: Optional[Union[Global[List[str]], Variable, Default[None]]] = None
    lease_time: Optional[Union[Global[int], Variable, Default[int]]] = Field(
        serialization_alias="leaseTime", validation_alias="leaseTime", default=Default[int](value=86400)
    )
    interface_mtu: Optional[Union[Global[int], Variable, Default[None]]] = Field(
        serialization_alias="interfaceMtu", validation_alias="interfaceMtu", default=None
    )
    domain_name: Optional[Union[Global[str], Variable, Default[None]]] = Field(
        serialization_alias="domainName", validation_alias="domainName", default=None
    )
    default_gateway: Optional[Union[Global[str], Variable, Default[None]]] = Field(
        serialization_alias="defaultGateway", validation_alias="defaultGateway", default=None
    )
    dns_servers: Optional[Union[Global[List[str]], Variable, Default[None]]] = Field(
        serialization_alias="dnsServers", validation_alias="dnsServers", default=None
    )
    tftp_servers: Optional[Union[Global[List[str]], Variable, Default[None]]] = Field(
        serialization_alias="tftpServers", validation_alias="tftpServers", default=None
    )
    static_lease: Optional[List[StaticLease]] = Field(
        serialization_alias="staticLease", validation_alias="staticLease", default=None
    )
    option_code: Optional[List[Union[OptionCodeAscii, OptionCodeHex, OptionCodeIP]]] = Field(
        serialization_alias="optionCode", validation_alias="optionCode", default=None
    )


class DhcpSeverCreationPayload(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    name: str
    description: Optional[str] = None
    data: DhcpServerData
    metadata: Optional[dict] = None



# Copyright 2023 Cisco Systems, Inc. and its affiliates

import unittest
from unittest.mock import patch
from urllib.error import HTTPError

from ciscoconfparse import CiscoConfParse  # type: ignore

from catalystwan.api.templates.cli_template import CLITemplate
from catalystwan.dataclasses import Device
from catalystwan.exceptions import TemplateTypeError
from catalystwan.utils.device_model import DeviceModel


class TestCLITemplate(unittest.TestCase):
    def setUp(self):
        self.config = """
        system\n
        host-name               host5\n
        system-ip               192.168.1.25\n
        site-id                 2\n
        admin-tech-on-failure\n
        no route-consistency-check\n
        no vrrp-advt-with-phymac\n
        sp-organization-name    "organization"\n
        organization-name       "organization"\n
        vbond vbond\n
        aaa\n
        auth-order      local radius tacacs\n
        usergroup basic\n
        task system read\n
        task interface read\n
        !\n
        usergroup netadmin\n
        """
        self.template = CiscoConfParse(self.config.splitlines())

    @patch("catalystwan.session.ManagerSession")
    def test_load_success(self, mock_session):
        # Arrange
        mock_session.get_json.return_value = {"configType": "file", "templateConfiguration": self.config}
        # Act
        temp = CLITemplate(template_name="test", template_description="test", device_model=DeviceModel.VEDGE)
        answer = temp.load(session=mock_session, id="temp_id")
        # Assert
        self.assertEqual(answer.ioscfg, self.template.ioscfg)

    @patch("catalystwan.session.ManagerSession")
    def test_load_raise(self, mock_session):
        # Arrange
        mock_session.get_json.return_value = {
            "templateName": "test",
            "configType": "template",
            "templateConfiguration": self.config,
        }
        # Act
        temp = CLITemplate(template_name="test", template_description="test", device_model=DeviceModel.VEDGE)

        def answer():
            return temp.load(session=mock_session, id="temp_id")

        # Assert
        self.assertRaises(TemplateTypeError, answer)

    @patch("catalystwan.session.ManagerSession")
    def test_load_running_success(self, mock_session):
        # Arrange
        mock_session.get_json.return_value = {"configType": "file", "config": self.config}
        device = Device(
            personality="vedge",
            uuid="mock_uuid",
            id="mock_ip",
            hostname="mock_host",
            reachability="reachable",
            local_system_ip="mock_ip",
            status="normal",
            memUsage=1.0,
            connected_vManages=["192.168.0.1"],
            model="vedge-cloud",
        )
        # Act
        temp = CLITemplate(template_name="test", template_description="test", device_model=DeviceModel.VEDGE)
        answer = temp.load_running(session=mock_session, device=device)
        # Assert
        self.assertEqual(answer.ioscfg, self.template.ioscfg)

    def test_generate_payload(self):
        # Arrange
        template = CLITemplate(
            template_name="test", template_description="test", device_model=DeviceModel.VEDGE, config=self.template
        )
        # Act
        answer = template.generate_payload()
        # Assert
        templateConfiguration = (
            "        system\n"
            "        host-name               host5\n"
            "        system-ip               192.168.1.25\n"
            "        site-id                 2\n"
            "        admin-tech-on-failure\n"
            "        no route-consistency-check\n"
            "        no vrrp-advt-with-phymac\n"
            '        sp-organization-name    "organization"\n'
            '        organization-name       "organization"\n'
            "        vbond vbond\n"
            "        aaa\n"
            "        auth-order      local radius tacacs\n"
            "        usergroup basic\n"
            "        task system read\n"
            "        task interface read\n"
            "        !\n"
            "        usergroup netadmin"
        )
        proper_answer = {
            "templateName": "test",
            "templateDescription": "test",
            "deviceType": "vedge-cloud",
            "templateConfiguration": templateConfiguration,
            "factoryDefault": False,
            "configType": "file",
        }
        self.assertEqual(answer, proper_answer)

    def test_generate_payload_cedge(self):
        # Arrange
        template = CLITemplate(
            template_name="test",
            template_description="test",
            device_model=DeviceModel.VEDGE_C8000V,
            config=self.template,
        )
        # Act
        answer = template.generate_payload()
        # Assert
        templateConfiguration = (
            "        system\n"
            "        host-name               host5\n"
            "        system-ip               192.168.1.25\n"
            "        site-id                 2\n"
            "        admin-tech-on-failure\n"
            "        no route-consistency-check\n"
            "        no vrrp-advt-with-phymac\n"
            '        sp-organization-name    "organization"\n'
            '        organization-name       "organization"\n'
            "        vbond vbond\n"
            "        aaa\n"
            "        auth-order      local radius tacacs\n"
            "        usergroup basic\n"
            "        task system read\n"
            "        task interface read\n"
            "        !\n"
            "        usergroup netadmin"
        )
        proper_answer = {
            "templateName": "test",
            "templateDescription": "test",
            "deviceType": "vedge-C8000V",
            "templateConfiguration": templateConfiguration,
            "factoryDefault": False,
            "configType": "file",
            "cliType": "device",
            "draftMode": False,
        }
        self.assertEqual(answer, proper_answer)

    @patch("catalystwan.session.ManagerSession")
    def test_update_suceess(self, mock_session):
        # Arrange
        mock_session.put.return_value = {"data": {"attachedDevices": []}}
        templateConfiguration = (
            "        system\n"
            "        host-name               host6\n"
            "        system-ip               192.168.1.26\n"
        )
        config = CiscoConfParse(templateConfiguration.splitlines())
        # Act
        template = CLITemplate(template_name="test", template_description="test", device_model=DeviceModel.VEDGE)
        result = template.update(session=mock_session, id="temp_id", config=config)
        # Assert
        self.assertTrue(result)

    @patch("catalystwan.session.ManagerSession")
    def test_update_template_failure(self, mock_session):
        # Arrange

        mock_session.put.side_effect = HTTPError("url", 400, "error_400", "msg", 1)

        templateConfiguration = (
            "        system\n"
            "        host-name               host6\n"
            "        system-ip               192.168.1.26\n"
        )
        config = CiscoConfParse(templateConfiguration.splitlines())
        template = CLITemplate(template_name="test", template_description="test", device_model=DeviceModel.VEDGE)

        # Act
        with self.assertRaises(HTTPError):
            template.update(session=mock_session, id="temp_id", config=config)

    def test_compare_template(self):
        # Arrange
        templateConfiguration1 = (
            "        system\n"
            "        host-name               host6\n"
            "        system-ip               192.168.1.26\n"
        )
        config1 = CiscoConfParse(templateConfiguration1.splitlines())
        templateConfiguration2 = (
            "        system\n"
            "        host-name               host6\n"
            "        system-ip               192.168.1.26\n"
        )
        config2 = CiscoConfParse(templateConfiguration2.splitlines())
        # Act
        result = CLITemplate(
            template_name="test", template_description="test", device_model=DeviceModel.VEDGE
        ).compare_template(config1, config2)
        # Assert
        self.assertEqual(result, "")



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Optional, Type, overload
from uuid import UUID

from catalystwan.api.task_status_api import Task
from catalystwan.endpoints.configuration.policy.definition.access_control_list import (
    AclPolicyGetResponse,
    ConfigurationPolicyAclDefinition,
)
from catalystwan.endpoints.configuration.policy.definition.access_control_list_ipv6 import (
    AclIPv6PolicyGetResponse,
    ConfigurationPolicyAclIPv6Definition,
)
from catalystwan.endpoints.configuration.policy.definition.control import (
    ConfigurationPolicyControlDefinition,
    ControlPolicyGetResponse,
)
from catalystwan.endpoints.configuration.policy.definition.device_access import (
    ConfigurationPolicyDeviceAccessDefinition,
    DeviceAccessPolicyGetResponse,
)
from catalystwan.endpoints.configuration.policy.definition.device_access_ipv6 import (
    ConfigurationPolicyDeviceAccessIPv6Definition,
    DeviceAccessIPv6PolicyGetResponse,
)
from catalystwan.endpoints.configuration.policy.definition.hub_and_spoke import (
    ConfigurationPolicyHubAndSpokeDefinition,
    HubAndSpokePolicyGetResponse,
)
from catalystwan.endpoints.configuration.policy.definition.mesh import (
    ConfigurationPolicyMeshDefinition,
    MeshPolicyGetResponse,
)
from catalystwan.endpoints.configuration.policy.definition.qos_map import (
    ConfigurationPolicyQoSMapDefinition,
    QoSMapPolicyGetResponse,
)
from catalystwan.endpoints.configuration.policy.definition.rewrite import (
    ConfigurationPolicyRewriteRuleDefinition,
    RewritePolicyGetResponse,
)
from catalystwan.endpoints.configuration.policy.definition.rule_set import (
    ConfigurationPolicyRuleSetDefinition,
    RuleSetGetResponse,
)
from catalystwan.endpoints.configuration.policy.definition.security_group import (
    ConfigurationPolicySecurityGroupDefinition,
    SecurityGroupGetResponse,
)
from catalystwan.endpoints.configuration.policy.definition.traffic_data import (
    ConfigurationPolicyDataDefinition,
    TrafficDataPolicy,
    TrafficDataPolicyGetResponse,
)
from catalystwan.endpoints.configuration.policy.definition.vpn_membership import (
    ConfigurationPolicyVPNMembershipGroupDefinition,
    VPNMembershipPolicyGetResponse,
)
from catalystwan.endpoints.configuration.policy.definition.zone_based_firewall import (
    ConfigurationPolicyZoneBasedFirewallDefinition,
    ZoneBasedFWPolicyGetResponse,
)
from catalystwan.endpoints.configuration.policy.list.app import AppListInfo, ConfigurationPolicyApplicationList
from catalystwan.endpoints.configuration.policy.list.app_probe import (
    AppProbeClassListInfo,
    ConfigurationPolicyAppProbeClassList,
)
from catalystwan.endpoints.configuration.policy.list.as_path import ASPathListInfo, ConfigurationPolicyASPathList
from catalystwan.endpoints.configuration.policy.list.class_map import (
    ClassMapListInfo,
    ConfigurationPolicyForwardingClassList,
)
from catalystwan.endpoints.configuration.policy.list.color import ColorListInfo, ConfigurationPolicyColorList
from catalystwan.endpoints.configuration.policy.list.community import (
    CommunityListInfo,
    ConfigurationPolicyCommunityList,
)
from catalystwan.endpoints.configuration.policy.list.data_ipv6_prefix import (
    ConfigurationPolicyDataIPv6PrefixList,
    DataIPv6PrefixListInfo,
)
from catalystwan.endpoints.configuration.policy.list.data_prefix import (
    ConfigurationPolicyDataPrefixList,
    DataPrefixListInfo,
)
from catalystwan.endpoints.configuration.policy.list.expanded_community import (
    ConfigurationPolicyExpandedCommunityList,
    ExpandedCommunityListInfo,
)
from catalystwan.endpoints.configuration.policy.list.fqdn import ConfigurationPolicyFQDNList, FQDNListInfo
from catalystwan.endpoints.configuration.policy.list.geo_location import (
    ConfigurationPolicyGeoLocationList,
    GeoLocationListInfo,
)
from catalystwan.endpoints.configuration.policy.list.ips_signature import (
    ConfigurationPolicyIPSSignatureList,
    IPSSignatureListInfo,
)
from catalystwan.endpoints.configuration.policy.list.ipv6_prefix import (
    ConfigurationPolicyIPv6PrefixList,
    IPv6PrefixListInfo,
)
from catalystwan.endpoints.configuration.policy.list.local_app import ConfigurationPolicyLocalAppList, LocalAppListInfo
from catalystwan.endpoints.configuration.policy.list.local_domain import (
    ConfigurationPolicyLocalDomainList,
    LocalDomainListInfo,
)
from catalystwan.endpoints.configuration.policy.list.mirror import ConfigurationPolicyMirrorList, MirrorListInfo
from catalystwan.endpoints.configuration.policy.list.policer import ConfigurationPolicyPolicerClassList, PolicerListInfo
from catalystwan.endpoints.configuration.policy.list.port import ConfigurationPolicyPortList, PortListInfo
from catalystwan.endpoints.configuration.policy.list.preferred_color_group import (
    ConfigurationPreferredColorGroupList,
    PreferredColorGroupListInfo,
)
from catalystwan.endpoints.configuration.policy.list.prefix import ConfigurationPolicyPrefixList, PrefixListInfo
from catalystwan.endpoints.configuration.policy.list.protocol_name import (
    ConfigurationPolicyProtocolNameList,
    ProtocolNameListInfo,
)
from catalystwan.endpoints.configuration.policy.list.region import ConfigurationPolicyRegionList, RegionListInfo
from catalystwan.endpoints.configuration.policy.list.site import ConfigurationPolicySiteList, SiteListInfo
from catalystwan.endpoints.configuration.policy.list.sla import ConfigurationPolicySLAClassList, SLAClassListInfo
from catalystwan.endpoints.configuration.policy.list.tloc import ConfigurationPolicyTLOCList, TLOCListInfo
from catalystwan.endpoints.configuration.policy.list.url_allow_list import (
    ConfigurationPolicyURLAllowList,
    URLAllowListInfo,
)
from catalystwan.endpoints.configuration.policy.list.url_block_list import (
    ConfigurationPolicyURLBlockList,
    URLBlockListInfo,
)
from catalystwan.endpoints.configuration.policy.list.vpn import ConfigurationPolicyVPNList, VPNListInfo
from catalystwan.endpoints.configuration.policy.list.zone import ConfigurationPolicyZoneList, ZoneListInfo
from catalystwan.endpoints.configuration.policy.security_template import ConfigurationSecurityTemplatePolicy
from catalystwan.endpoints.configuration.policy.vedge_template import ConfigurationVEdgeTemplatePolicy
from catalystwan.endpoints.configuration.policy.vsmart_template import (
    ConfigurationVSmartTemplatePolicy,
    VSmartConnectivityStatus,
)
from catalystwan.models.misc.application_protocols import ApplicationProtocol
from catalystwan.models.policy import AnyPolicyDefinition, AnyPolicyList
from catalystwan.models.policy.centralized import CentralizedPolicy, CentralizedPolicyEditPayload, CentralizedPolicyInfo
from catalystwan.models.policy.definitions.access_control_list import AclPolicy
from catalystwan.models.policy.definitions.access_control_list_ipv6 import AclIPv6Policy
from catalystwan.models.policy.definitions.control import ControlPolicy
from catalystwan.models.policy.definitions.device_access import DeviceAccessPolicy
from catalystwan.models.policy.definitions.device_access_ipv6 import DeviceAccessIPv6Policy
from catalystwan.models.policy.definitions.hub_and_spoke import HubAndSpokePolicy
from catalystwan.models.policy.definitions.mesh import MeshPolicy
from catalystwan.models.policy.definitions.qos_map import QoSMapPolicy
from catalystwan.models.policy.definitions.rewrite import RewritePolicy
from catalystwan.models.policy.definitions.rule_set import RuleSet
from catalystwan.models.policy.definitions.security_group import SecurityGroup
from catalystwan.models.policy.definitions.vpn_membership import VPNMembershipPolicy
from catalystwan.models.policy.definitions.zone_based_firewall import ZoneBasedFWPolicy
from catalystwan.models.policy.lists import (
    AppList,
    AppProbeClassList,
    ASPathList,
    ClassMapList,
    ColorList,
    CommunityList,
    DataIPv6PrefixList,
    DataPrefixList,
    ExpandedCommunityList,
    FQDNList,
    GeoLocationList,
    IPSSignatureList,
    IPv6PrefixList,
    LocalAppList,
    LocalDomainList,
    MirrorList,
    PolicerList,
    PolicyListBase,
    PortList,
    PreferredColorGroupList,
    PrefixList,
    ProtocolNameList,
    RegionList,
    SiteList,
    SLAClassList,
    TLOCList,
    URLAllowList,
    URLBlockList,
    VPNList,
    ZoneList,
)
from catalystwan.models.policy.localized import (
    LocalizedPolicy,
    LocalizedPolicyDeviceInfo,
    LocalizedPolicyEditResponse,
    LocalizedPolicyInfo,
)
from catalystwan.models.policy.policy_definition import (
    PolicyDefinitionBase,
    PolicyDefinitionEditResponse,
    PolicyDefinitionEndpoints,
    PolicyDefinitionInfo,
)
from catalystwan.models.policy.policy_list import PolicyListEndpoints
from catalystwan.models.policy.security import (
    AnySecurityPolicy,
    AnySecurityPolicyInfo,
    SecurityPolicy,
    SecurityPolicyEditResponse,
    UnifiedSecurityPolicy,
)
from catalystwan.typed_list import DataSequence

if TYPE_CHECKING:
    from catalystwan.session import ManagerSession


POLICY_LIST_ENDPOINTS_MAP: Mapping[type, type] = {
    AppList: ConfigurationPolicyApplicationList,
    AppProbeClassList: ConfigurationPolicyAppProbeClassList,
    ASPathList: ConfigurationPolicyASPathList,
    ClassMapList: ConfigurationPolicyForwardingClassList,
    ColorList: ConfigurationPolicyColorList,
    CommunityList: ConfigurationPolicyCommunityList,
    DataIPv6PrefixList: ConfigurationPolicyDataIPv6PrefixList,
    DataPrefixList: ConfigurationPolicyDataPrefixList,
    ExpandedCommunityList: ConfigurationPolicyExpandedCommunityList,
    FQDNList: ConfigurationPolicyFQDNList,
    GeoLocationList: ConfigurationPolicyGeoLocationList,
    IPSSignatureList: ConfigurationPolicyIPSSignatureList,
    IPv6PrefixList: ConfigurationPolicyIPv6PrefixList,
    LocalAppList: ConfigurationPolicyLocalAppList,
    LocalDomainList: ConfigurationPolicyLocalDomainList,
    MirrorList: ConfigurationPolicyMirrorList,
    PolicerList: ConfigurationPolicyPolicerClassList,
    PortList: ConfigurationPolicyPortList,
    PreferredColorGroupList: ConfigurationPreferredColorGroupList,
    PrefixList: ConfigurationPolicyPrefixList,
    ProtocolNameList: ConfigurationPolicyProtocolNameList,
    RegionList: ConfigurationPolicyRegionList,
    SiteList: ConfigurationPolicySiteList,
    SLAClassList: ConfigurationPolicySLAClassList,
    TLOCList: ConfigurationPolicyTLOCList,
    URLBlockList: ConfigurationPolicyURLBlockList,
    URLAllowList: ConfigurationPolicyURLAllowList,
    VPNList: ConfigurationPolicyVPNList,
    ZoneList: ConfigurationPolicyZoneList,
}

POLICY_DEFINITION_ENDPOINTS_MAP: Mapping[type, type] = {
    RuleSet: ConfigurationPolicyRuleSetDefinition,
    SecurityGroup: ConfigurationPolicySecurityGroupDefinition,
    ZoneBasedFWPolicy: ConfigurationPolicyZoneBasedFirewallDefinition,
    TrafficDataPolicy: ConfigurationPolicyDataDefinition,
    QoSMapPolicy: ConfigurationPolicyQoSMapDefinition,
    RewritePolicy: ConfigurationPolicyRewriteRuleDefinition,
    ControlPolicy: ConfigurationPolicyControlDefinition,
    VPNMembershipPolicy: ConfigurationPolicyVPNMembershipGroupDefinition,
    HubAndSpokePolicy: ConfigurationPolicyHubAndSpokeDefinition,
    MeshPolicy: ConfigurationPolicyMeshDefinition,
    AclPolicy: ConfigurationPolicyAclDefinition,
    AclIPv6Policy: ConfigurationPolicyAclIPv6Definition,
    DeviceAccessPolicy: ConfigurationPolicyDeviceAccessDefinition,
    DeviceAccessIPv6Policy: ConfigurationPolicyDeviceAccessIPv6Definition,
}


class CentralizedPolicyAPI:
    def __init__(self, session: ManagerSession):
        self._session = session
        self._endpoints = ConfigurationVSmartTemplatePolicy(session)

    def activate(self, id: UUID) -> Task:
        task_id = self._endpoints.activate_policy(id).id
        return Task(self._session, task_id)

    def deactivate(self, id: UUID) -> Task:
        task_id = self._endpoints.deactivate_policy(id).id
        return Task(self._session, task_id)

    def create(self, policy: CentralizedPolicy) -> UUID:
        return self._endpoints.create_vsmart_template(policy).policy_id

    def edit(self, policy: CentralizedPolicyEditPayload, lock_checks: bool = True) -> None:
        if lock_checks:
            self._endpoints.edit_vsmart_template(policy.policy_id, policy)
        self._endpoints.edit_template_without_lock_checks(policy.policy_id, policy)

    def delete(self, id: UUID) -> None:
        self._endpoints.delete_vsmart_template(id)

    @overload
    def get(self) -> DataSequence[CentralizedPolicyInfo]:
        ...

    @overload
    def get(self, id: UUID) -> CentralizedPolicy:
        ...

    def get(self, id: Optional[UUID] = None) -> Any:
        if id is not None:
            return self._endpoints.get_template_by_policy_id(id)
        return self._endpoints.generate_vsmart_policy_template_list()

    def check_vsmart_connectivity(self) -> DataSequence[VSmartConnectivityStatus]:
        return self._endpoints.check_vsmart_connectivity_status()


class LocalizedPolicyAPI:
    def __init__(self, session: ManagerSession):
        self._session = session
        self._endpoints = ConfigurationVEdgeTemplatePolicy(session)

    def create(self, policy: LocalizedPolicy) -> UUID:
        return self._endpoints.create_vedge_template(policy).policy_id

    def edit(self, id: UUID, policy: LocalizedPolicy) -> LocalizedPolicyEditResponse:
        return self._endpoints.edit_vedge_template(id, policy)

    def delete(self, id: UUID) -> None:
        self._endpoints.delete_vedge_template(id)

    @overload
    def get(self) -> DataSequence[LocalizedPolicyInfo]:
        ...

    @overload
    def get(self, id: UUID) -> LocalizedPolicy:
        ...

    def get(self, id: Optional[UUID] = None) -> Any:
        if id is not None:
            return self._endpoints.get_vedge_template(id)
        return self._endpoints.generate_policy_template_list()

    def list_devices(self, id: Optional[UUID] = None) -> DataSequence[LocalizedPolicyDeviceInfo]:
        if id is not None:
            return self._endpoints.get_device_list_by_policy(id)
        return self._endpoints.get_vedge_policy_device_list()

    def preview(self, id: UUID) -> str:
        return self._endpoints.preview_by_id(id).preview


class SecurityPolicyAPI:
    def __init__(self, session: ManagerSession):
        self._session = session
        self._endpoints = ConfigurationSecurityTemplatePolicy(session)

    def create(self, policy: AnySecurityPolicy) -> UUID:
        # POST does not return anything! we need to list all after creation and find by name to get id
        self._endpoints.create_security_template(policy)
        policy_infos = [
            info.root
            for info in self._endpoints.generate_security_template_list()
            if info.root.policy_name == policy.policy_name
        ]
        assert len(policy_infos) == 1
        return policy_infos[0].policy_id

    def edit(self, id: UUID, policy: AnySecurityPolicy) -> SecurityPolicyEditResponse:
        return self._endpoints.edit_security_template(id, policy)

    def delete(self, id: UUID) -> None:
        self._endpoints.delete_security_template(id)

    @overload
    def get(self) -> List[AnySecurityPolicyInfo]:
        ...

    @overload
    def get(self, id: UUID) -> AnySecurityPolicy:
        ...

    def get(self, id: Optional[UUID] = None) -> Any:
        if id is not None:
            return self._endpoints.get_security_template(id).root
        return [info.root for info in self._endpoints.generate_security_template_list()]


class PolicyListsAPI:
    def __init__(self, session: ManagerSession):
        self._session = session

    def __get_list_endpoints_instance(self, payload_type: type) -> PolicyListEndpoints:
        endpoints_class = POLICY_LIST_ENDPOINTS_MAP.get(payload_type)
        if endpoints_class is None:
            raise TypeError(f"Unsupported policy list type: {payload_type}")
        return endpoints_class(self._session)

    def create(self, policy_list: AnyPolicyList) -> UUID:
        endpoints = self.__get_list_endpoints_instance(type(policy_list))
        return endpoints.create_policy_list(payload=policy_list).list_id

    def edit(self, id: UUID, policy_list: AnyPolicyList) -> None:
        endpoints = self.__get_list_endpoints_instance(type(policy_list))
        endpoints.edit_policy_list(id=id, payload=policy_list)

    def delete(self, type: Type[AnyPolicyList], id: UUID) -> None:
        endpoints = self.__get_list_endpoints_instance(type)
        endpoints.delete_policy_list(id=id)

    @overload
    def get(self, type: Type[AppList]) -> DataSequence[AppListInfo]:
        ...

    @overload
    def get(self, type: Type[AppProbeClassList]) -> DataSequence[AppProbeClassListInfo]:
        ...

    @overload
    def get(self, type: Type[ASPathList]) -> DataSequence[ASPathListInfo]:
        ...

    @overload
    def get(self, type: Type[ClassMapList]) -> DataSequence[ClassMapListInfo]:
        ...

    @overload
    def get(self, type: Type[ColorList]) -> DataSequence[ColorListInfo]:
        ...

    @overload
    def get(self, type: Type[CommunityList]) -> DataSequence[CommunityListInfo]:
        ...

    @overload
    def get(self, type: Type[DataIPv6PrefixList]) -> DataSequence[DataIPv6PrefixListInfo]:
        ...

    @overload
    def get(self, type: Type[DataPrefixList]) -> DataSequence[DataPrefixListInfo]:
        ...

    @overload
    def get(self, type: Type[ExpandedCommunityList]) -> DataSequence[ExpandedCommunityListInfo]:
        ...

    @overload
    def get(self, type: Type[FQDNList]) -> DataSequence[FQDNListInfo]:
        ...

    @overload
    def get(self, type: Type[GeoLocationList]) -> DataSequence[GeoLocationListInfo]:
        ...

    @overload
    def get(self, type: Type[IPSSignatureList]) -> DataSequence[IPSSignatureListInfo]:
        ...

    @overload
    def get(self, type: Type[IPv6PrefixList]) -> DataSequence[IPv6PrefixListInfo]:
        ...

    @overload
    def get(self, type: Type[LocalAppList]) -> DataSequence[LocalAppListInfo]:
        ...

    @overload
    def get(self, type: Type[LocalDomainList]) -> DataSequence[LocalDomainListInfo]:
        ...

    @overload
    def get(self, type: Type[MirrorList]) -> DataSequence[MirrorListInfo]:
        ...

    @overload
    def get(self, type: Type[PolicerList]) -> DataSequence[PolicerListInfo]:
        ...

    @overload
    def get(self, type: Type[PortList]) -> DataSequence[PortListInfo]:
        ...

    @overload
    def get(self, type: Type[PreferredColorGroupList]) -> DataSequence[PreferredColorGroupListInfo]:
        ...

    @overload
    def get(self, type: Type[PrefixList]) -> DataSequence[PrefixListInfo]:
        ...

    @overload
    def get(self, type: Type[ProtocolNameList]) -> DataSequence[ProtocolNameListInfo]:
        ...

    @overload
    def get(self, type: Type[RegionList]) -> DataSequence[RegionListInfo]:
        ...

    @overload
    def get(self, type: Type[SiteList]) -> DataSequence[SiteListInfo]:
        ...

    @overload
    def get(self, type: Type[SLAClassList]) -> DataSequence[SLAClassListInfo]:
        ...

    @overload
    def get(self, type: Type[TLOCList]) -> DataSequence[TLOCListInfo]:
        ...

    @overload
    def get(self, type: Type[URLBlockList]) -> DataSequence[URLBlockListInfo]:
        ...

    @overload
    def get(self, type: Type[URLAllowList]) -> DataSequence[URLAllowListInfo]:
        ...

    @overload
    def get(self, type: Type[VPNList]) -> DataSequence[VPNListInfo]:
        ...

    @overload
    def get(self, type: Type[ZoneList]) -> DataSequence[ZoneListInfo]:
        ...

    # get by id

    @overload
    def get(self, type: Type[AppList], id: UUID) -> AppListInfo:
        ...

    @overload
    def get(self, type: Type[AppProbeClassList], id: UUID) -> AppProbeClassListInfo:
        ...

    @overload
    def get(self, type: Type[ASPathList], id: UUID) -> ASPathListInfo:
        ...

    @overload
    def get(self, type: Type[ClassMapList], id: UUID) -> ClassMapListInfo:
        ...

    @overload
    def get(self, type: Type[ColorList], id: UUID) -> ColorListInfo:
        ...

    @overload
    def get(self, type: Type[CommunityList], id: UUID) -> CommunityListInfo:
        ...

    @overload
    def get(self, type: Type[DataIPv6PrefixList], id: UUID) -> DataIPv6PrefixListInfo:
        ...

    @overload
    def get(self, type: Type[DataPrefixList], id: UUID) -> DataPrefixListInfo:
        ...

    @overload
    def get(self, type: Type[ExpandedCommunityList], id: UUID) -> ExpandedCommunityListInfo:
        ...

    @overload
    def get(self, type: Type[FQDNList], id: UUID) -> FQDNListInfo:
        ...

    @overload
    def get(self, type: Type[GeoLocationList], id: UUID) -> GeoLocationListInfo:
        ...

    @overload
    def get(self, type: Type[IPSSignatureList], id: UUID) -> IPSSignatureListInfo:
        ...

    @overload
    def get(self, type: Type[IPv6PrefixList], id: UUID) -> IPv6PrefixListInfo:
        ...

    @overload
    def get(self, type: Type[LocalAppList], id: UUID) -> LocalAppListInfo:
        ...

    @overload
    def get(self, type: Type[LocalDomainList], id: UUID) -> LocalDomainListInfo:
        ...

    @overload
    def get(self, type: Type[MirrorList], id: UUID) -> MirrorListInfo:
        ...

    @overload
    def get(self, type: Type[PolicerList], id: UUID) -> PolicerListInfo:
        ...

    @overload
    def get(self, type: Type[PortList], id: UUID) -> PortListInfo:
        ...

    @overload
    def get(self, type: Type[PreferredColorGroupList], id: UUID) -> PreferredColorGroupListInfo:
        ...

    @overload
    def get(self, type: Type[PrefixList], id: UUID) -> PrefixListInfo:
        ...

    @overload
    def get(self, type: Type[ProtocolNameList], id: UUID) -> ProtocolNameListInfo:
        ...

    @overload
    def get(self, type: Type[RegionList], id: UUID) -> RegionListInfo:
        ...

    @overload
    def get(self, type: Type[SiteList], id: UUID) -> SiteListInfo:
        ...

    @overload
    def get(self, type: Type[SLAClassList], id: UUID) -> SLAClassListInfo:
        ...

    @overload
    def get(self, type: Type[TLOCList], id: UUID) -> TLOCListInfo:
        ...

    @overload
    def get(self, type: Type[URLBlockList], id: UUID) -> URLBlockListInfo:
        ...

    @overload
    def get(self, type: Type[URLAllowList], id: UUID) -> URLAllowListInfo:
        ...

    @overload
    def get(self, type: Type[VPNList], id: UUID) -> VPNListInfo:
        ...

    @overload
    def get(self, type: Type[ZoneList], id: UUID) -> ZoneListInfo:
        ...

    def get(self, type: Type[AnyPolicyList], id: Optional[UUID] = None) -> Any:
        endpoints = self.__get_list_endpoints_instance(type)
        if id is not None:
            return endpoints.get_lists_by_id(id=id)
        return endpoints.get_policy_lists()


class PolicyDefinitionsAPI:
    def __init__(self, session: ManagerSession):
        self._session = session

    def __get_definition_endpoints_instance(self, payload_type: type) -> PolicyDefinitionEndpoints:
        endpoints_class = POLICY_DEFINITION_ENDPOINTS_MAP.get(payload_type)
        if endpoints_class is None:
            raise TypeError(f"Unsupported policy definition type: {payload_type}")
        return endpoints_class(self._session)

    def create(self, policy_definition: AnyPolicyDefinition) -> UUID:
        endpoints = self.__get_definition_endpoints_instance(type(policy_definition))
        return endpoints.create_policy_definition(payload=policy_definition).definition_id

    def edit(self, id: UUID, policy_definition: AnyPolicyDefinition) -> PolicyDefinitionEditResponse:
        endpoints = self.__get_definition_endpoints_instance(type(policy_definition))
        return endpoints.edit_policy_definition(id=id, payload=policy_definition)

    def delete(self, type: Type[AnyPolicyDefinition], id: UUID) -> None:
        endpoints = self.__get_definition_endpoints_instance(type)
        endpoints.delete_policy_definition(id=id)

    @overload
    def get(self, type: Type[TrafficDataPolicy]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @overload
    def get(self, type: Type[RuleSet]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @overload
    def get(self, type: Type[SecurityGroup]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @overload
    def get(self, type: Type[ZoneBasedFWPolicy]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @overload
    def get(self, type: Type[QoSMapPolicy]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @overload
    def get(self, type: Type[RewritePolicy]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @overload
    def get(self, type: Type[ControlPolicy]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @overload
    def get(self, type: Type[VPNMembershipPolicy]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @overload
    def get(self, type: Type[HubAndSpokePolicy]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @overload
    def get(self, type: Type[MeshPolicy]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @overload
    def get(self, type: Type[AclPolicy]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @overload
    def get(self, type: Type[AclIPv6Policy]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @overload
    def get(self, type: Type[DeviceAccessPolicy]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @overload
    def get(self, type: Type[DeviceAccessIPv6Policy]) -> DataSequence[PolicyDefinitionInfo]:
        ...

    # get by id

    @overload
    def get(self, type: Type[TrafficDataPolicy], id: UUID) -> TrafficDataPolicyGetResponse:
        ...

    @overload
    def get(self, type: Type[RuleSet], id: UUID) -> RuleSetGetResponse:
        ...

    @overload
    def get(self, type: Type[SecurityGroup], id: UUID) -> SecurityGroupGetResponse:
        ...

    @overload
    def get(self, type: Type[ZoneBasedFWPolicy], id: UUID) -> ZoneBasedFWPolicyGetResponse:
        ...

    @overload
    def get(self, type: Type[QoSMapPolicy], id: UUID) -> QoSMapPolicyGetResponse:
        ...

    @overload
    def get(self, type: Type[RewritePolicy], id: UUID) -> RewritePolicyGetResponse:
        ...

    @overload
    def get(self, type: Type[ControlPolicy], id: UUID) -> ControlPolicyGetResponse:
        ...

    @overload
    def get(self, type: Type[VPNMembershipPolicy], id: UUID) -> VPNMembershipPolicyGetResponse:
        ...

    @overload
    def get(self, type: Type[HubAndSpokePolicy], id: UUID) -> HubAndSpokePolicyGetResponse:
        ...

    @overload
    def get(self, type: Type[MeshPolicy], id: UUID) -> MeshPolicyGetResponse:
        ...

    @overload
    def get(self, type: Type[AclPolicy], id: UUID) -> AclPolicyGetResponse:
        ...

    @overload
    def get(self, type: Type[AclIPv6Policy], id: UUID) -> AclIPv6PolicyGetResponse:
        ...

    @overload
    def get(self, type: Type[DeviceAccessPolicy], id: UUID) -> DeviceAccessPolicyGetResponse:
        ...

    @overload
    def get(self, type: Type[DeviceAccessIPv6Policy], id: UUID) -> DeviceAccessIPv6PolicyGetResponse:
        ...

    def get(self, type: Type[AnyPolicyDefinition], id: Optional[UUID] = None) -> Any:
        endpoints = self.__get_definition_endpoints_instance(type)
        if id is not None:
            return endpoints.get_policy_definition(id=id)
        return endpoints.get_definitions()


class PolicyAPI:
    """This is exposing so called 'UX 1.0' API"""

    def __init__(self, session: ManagerSession):
        self._session = session
        self.centralized = CentralizedPolicyAPI(session)
        self.localized = LocalizedPolicyAPI(session)
        self.security = SecurityPolicyAPI(session)
        self.definitions = PolicyDefinitionsAPI(session)
        self.lists = PolicyListsAPI(session)

    def delete_any(self, _type: Any, id: UUID) -> None:
        if issubclass(_type, PolicyListBase):
            self.lists.delete(_type, id)
        elif issubclass(_type, PolicyDefinitionBase):
            self.definitions.delete(_type, id)
        elif _type == CentralizedPolicy:
            self.centralized.delete(id)
        elif _type == LocalizedPolicy:
            self.localized.delete(id)
        elif _type in [SecurityPolicy, UnifiedSecurityPolicy]:
            self.security.delete(id)
        else:
            raise TypeError(f"Cannot find API method to delete item type: {_type}, {id}")

    def get_protocol_map(self) -> Dict[str, ApplicationProtocol]:
        result = {}
        protocol_map_list = self._session.endpoints.misc.get_application_protocols()
        for protocol_map in protocol_map_list:
            result.update(protocol_map.root)
        return result



# Copyright 2024 Cisco Systems, Inc. and its affiliates

# mypy: disable-error-code="empty-body"
from uuid import UUID

from catalystwan.endpoints import APIEndpoints, delete, get, post, put
from catalystwan.models.policy.definitions.access_control_list_ipv6 import AclIPv6Policy
from catalystwan.models.policy.policy_definition import (
    PolicyDefinitionEditResponse,
    PolicyDefinitionEndpoints,
    PolicyDefinitionGetResponse,
    PolicyDefinitionId,
    PolicyDefinitionInfo,
    PolicyDefinitionPreview,
)
from catalystwan.typed_list import DataSequence


class AclIPv6PolicyEditPayload(AclIPv6Policy, PolicyDefinitionId):
    pass


class AclIPv6PolicyGetResponse(AclIPv6Policy, PolicyDefinitionGetResponse):
    pass


class ConfigurationPolicyAclIPv6Definition(APIEndpoints, PolicyDefinitionEndpoints):
    @post("/template/policy/definition/aclv6")
    def create_policy_definition(self, payload: AclIPv6Policy) -> PolicyDefinitionId:
        ...

    @delete("/template/policy/definition/aclv6/{id}")
    def delete_policy_definition(self, id: UUID) -> None:
        ...

    def edit_multiple_policy_definition(self):
        # PUT /template/policy/definition/aclv6/multiple/{id}
        ...

    @put("/template/policy/definition/aclv6/{id}")
    def edit_policy_definition(self, id: UUID, payload: AclIPv6PolicyEditPayload) -> PolicyDefinitionEditResponse:
        ...

    @get("/template/policy/definition/aclv6", "data")
    def get_definitions(self) -> DataSequence[PolicyDefinitionInfo]:
        ...

    @get("/template/policy/definition/aclv6/{id}")
    def get_policy_definition(self, id: UUID) -> AclIPv6PolicyGetResponse:
        ...

    @post("/template/policy/definition/aclv6/preview")
    def preview_policy_definition(self, payload: AclIPv6Policy) -> PolicyDefinitionPreview:
        ...

    @get("/template/policy/definition/aclv6/preview/{id}")
    def preview_policy_definition_by_id(self, id: UUID) -> PolicyDefinitionPreview:
        ...

    def save_policy_definition_in_bulk(self):
        # PUT /template/policy/definition/aclv6/bulk
        ...



# Copyright 2023 Cisco Systems, Inc. and its affiliates

# mypy: disable-error-code="empty-body"
from typing import List, Optional

from pydantic import BaseModel, ConfigDict, Field

from catalystwan.endpoints import APIEndpoints, get
from catalystwan.typed_list import DataSequence


class TLOC(BaseModel):
    color: str
    encapsulation: str


class Tier(BaseModel):
    """Endpoint: /dataservice/tier

    Since vManage 20.12 version, object has been renamed to "Resource Profile".
    """

    model_config = ConfigDict(populate_by_name=True)
    name: str = Field(serialization_alias="tierName", validation_alias="tierName")
    vpn: int
    rid: int = Field(serialization_alias="@rid", validation_alias="@rid")
    ipv4_route_limit_type: Optional[str] = Field(
        default=None, serialization_alias="ipv4RouteLimitType", validation_alias="ipv4RouteLimitType"
    )
    ipv4_route_limit_threshold: Optional[int] = Field(
        default=None, serialization_alias="ipv4RouteLimitThreshold", validation_alias="ipv4RouteLimitThreshold"
    )
    ipv4_route_limit: Optional[int] = Field(
        default=None, serialization_alias="ipv4RouteLimit", validation_alias="ipv4RouteLimit"
    )
    ipv6_route_limit_type: Optional[str] = Field(
        default=None, serialization_alias="ipv6RouteLimitType", validation_alias="ipv6RouteLimitType"
    )
    ipv6_route_limit_threshold: Optional[int] = Field(
        default=None, serialization_alias="ipv6RouteLimitThreshold", validation_alias="ipv6RouteLimitThreshold"
    )
    ipv6_route_limit: Optional[int] = Field(
        default=None, serialization_alias="ipv6RouteLimit", validation_alias="ipv6RouteLimit"
    )
    tlocs: List[TLOC] = Field(default=[])
    # New in 20.12 version
    nat_session_limit: Optional[int] = Field(
        default=None, serialization_alias="natSessionLimit", validation_alias="natSessionLimit"
    )


class DeviceData(BaseModel):
    # Field "model_sku" has conflict with protected namespace "model_"
    model_config = ConfigDict(populate_by_name=True, protected_namespaces=())
    board_serial: Optional[str] = Field(
        default=None, serialization_alias="board-serial", validation_alias="board-serial"
    )
    certificate_validity: Optional[str] = Field(
        default=None, serialization_alias="certificate-validity", validation_alias="certificate-validity"
    )
    connected_vmanages: Optional[List[str]] = Field(
        default=None, serialization_alias="connectedVManages", validation_alias="connectedVManages"
    )
    control_connections: Optional[str] = Field(
        default=None, serialization_alias="controlConnections", validation_alias="controlConnections"
    )
    device_groups: Optional[List[str]] = Field(
        default=None, serialization_alias="device-groups", validation_alias="device-groups"
    )
    device_model: Optional[str] = Field(
        default=None, serialization_alias="device-model", validation_alias="device-model"
    )
    device_os: Optional[str] = Field(default=None, serialization_alias="device-os", validation_alias="device-os")
    device_type: Optional[str] = Field(default=None, serialization_alias="device-type", validation_alias="device-type")
    device_id: str = Field(serialization_alias="deviceId", validation_alias="deviceId")
    domain_id: Optional[str] = Field(default=None, serialization_alias="domain-id", validation_alias="domain-id")
    host_name: Optional[str] = Field(default=None, serialization_alias="host-name", validation_alias="host-name")
    is_device_geo_data: Optional[bool] = Field(
        default=None, serialization_alias="isDeviceGeoData", validation_alias="isDeviceGeoData"
    )
    lastupdated: Optional[int] = None
    latitude: Optional[str] = None
    layout_level: Optional[int] = Field(default=None, serialization_alias="layoutLevel", validation_alias="layoutLevel")
    local_system_ip: Optional[str] = Field(
        default=None, serialization_alias="local-system-ip", validation_alias="local-system-ip"
    )
    longitude: Optional[str] = None
    max_controllers: Optional[str] = Field(
        default=None, serialization_alias="max-controllers", validation_alias="max-controllers"
    )
    model_sku: Optional[str] = None
    personality: Optional[str] = None
    platform: Optional[str] = None
    reachability: Optional[str] = None
    site_id: Optional[str] = Field(default=None, serialization_alias="site-id", validation_alias="site-id")
    state: Optional[str] = None
    state_description: Optional[str] = None
    status: Optional[str] = None
    status_order: Optional[int] = Field(default=None, serialization_alias="statusOrder", validation_alias="statusOrder")
    system_ip: Optional[str] = Field(default=None, serialization_alias="system-ip", validation_alias="system-ip")
    testbed_mode: Optional[bool] = None
    timezone: Optional[str] = None
    total_cpu_count: Optional[str] = None
    uptime_date: Optional[int] = Field(default=None, serialization_alias="uptime-date", validation_alias="uptime-date")
    uuid: Optional[str] = None
    validity: Optional[str] = None
    version: Optional[str] = None


class MonitoringDeviceDetails(APIEndpoints):
    def add_tier(self):
        #  POST /device/tier
        ...

    def delete_tier(self):
        #  DELETE /device/tier/{tierName}
        ...

    def enable_sdavcon_device(self):
        #  POST /device/enableSDAVC/{deviceIP}/{enable}
        ...

    def generate_device_state_data(self):
        #  GET /data/device/state/{state_data_type}
        ...

    def generate_device_state_data_fields(self):
        #  GET /data/device/state/{state_data_type}/fields
        ...

    def generate_device_state_data_with_query_string(self):
        #  GET /data/device/state/{state_data_type}/query
        ...

    def get_all_device_status(self):
        #  GET /device/status
        ...

    def get_device_counters(self):
        #  GET /device/counters
        ...

    def get_device_list_as_key_value(self):
        #  GET /device/keyvalue
        ...

    def get_device_models(self):
        #  GET /device/models/{uuid}
        ...

    def get_device_only_status(self):
        #  GET /device/devicestatus
        ...

    def get_device_running_config(self):
        #  GET /device/config
        ...

    def get_device_running_config_html(self):
        #  GET /device/config/html
        ...

    def get_device_tloc_status(self):
        #  GET /device/tloc
        ...

    def get_device_tloc_util(self):
        #  GET /device/tlocutil
        ...

    def get_device_tloc_util_details(self):
        #  GET /device/tlocutil/detail
        ...

    def get_hardware_health_details(self):
        #  GET /device/hardwarehealth/detail
        ...

    def get_hardware_health_summary(self):
        #  GET /device/hardwarehealth/summary
        ...

    def get_stats_queues(self):
        #  GET /device/stats
        ...

    def get_sync_queues(self):
        #  GET /device/queues
        ...

    @get("/device/tier", "data")
    def get_tiers(self) -> DataSequence[Tier]:
        ...

    def get_unconfigured(self):
        #  GET /device/unconfigured
        ...

    def get_vmanage_system_ip(self):
        #  GET /device/vmanage
        ...

    def get_vedge_inventory(self):
        #  GET /device/vedgeinventory/detail
        ...

    def get_vedge_inventory_summary(self):
        #  GET /device/vedgeinventory/summary
        ...

    def list_all_device_models(self):
        #  GET /device/models
        ...

    @get("/device", "data")
    def list_all_devices(self) -> DataSequence[DeviceData]:
        #  GET /device
        ...

    def list_all_monitor_details_devices(self):
        #  GET /device/monitor
        ...

    def list_currently_syncing_devices(self):
        #  GET /device/sync_status
        ...

    def list_reachable_devices(self):
        #  GET /device/reachable
        ...

    def list_unreachable_devices(self):
        #  GET /device/unreachable
        ...

    def remove_unreachable_device(self):
        #  DELETE /device/unreachable/{deviceIP}
        ...

    def set_block_sync(self):
        #  POST /device/blockSync
        ...

    def sync_all_devices_mem_db(self):
        #  POST /device/syncall/memorydb
        ...



# Copyright 2022 Cisco Systems, Inc. and its affiliates

import datetime as dt
from typing import List, Optional

from attr import define, field  # type: ignore
from pydantic import BaseModel, ConfigDict, Field

from catalystwan.exceptions import RetrieveIntervalOutOfRange
from catalystwan.utils.alarm_status import Severity
from catalystwan.utils.certificate_status import ValidityPeriod
from catalystwan.utils.colors import PrintColors
from catalystwan.utils.creation_tools import FIELD_NAME, asdict, convert_attributes
from catalystwan.utils.personality import Personality
from catalystwan.utils.reachability import Reachability
from catalystwan.utils.template_type import TemplateType


class DataclassBase:
    def __str__(self):
        return (
            f"{self.__class__.__name__}(\n"
            + "\n".join(f"    {attribute[0]}: {attribute[1]}, " for attribute in asdict(self).items())
            + "\n)"
        )


@define(frozen=True, field_transformer=convert_attributes)
class AdminTech(DataclassBase):
    creation_time: dt.datetime = field(metadata={FIELD_NAME: "creationTime"})
    size: int
    filename: str = field(metadata={FIELD_NAME: "fileName"})
    state: str
    device_ip: str = field(metadata={FIELD_NAME: "deviceIP"})
    system_ip: str = field(metadata={FIELD_NAME: "local-system-ip"})
    token_id: str = field(metadata={FIELD_NAME: "requestTokenId"})
    tac_state: Optional[str] = field(default=None)


@define(frozen=True, field_transformer=convert_attributes)
class DeviceAdminTech(DataclassBase):
    filename: str = field(metadata={FIELD_NAME: "fileName"})
    creation_time: dt.datetime = field(metadata={FIELD_NAME: "creationTime"})
    size: int
    state: str
    token_id: Optional[str] = field(default=None, metadata={FIELD_NAME: "requestTokenId"})


@define(frozen=True, field_transformer=convert_attributes)
class AlarmData(DataclassBase):
    severity: Severity = field(converter=Severity, default=None)
    component: Optional[str] = field(default=None)
    active: Optional[bool] = field(default=None)
    severity_number: Optional[int] = field(default=None)
    name: Optional[str] = field(default=None, metadata={FIELD_NAME: "type"})
    system_ip: Optional[str] = field(default=None, metadata={FIELD_NAME: "system-ip"})
    hostname: Optional[str] = field(default=None, metadata={FIELD_NAME: "host-name"})
    site_id: Optional[str] = field(default=None, metadata={FIELD_NAME: "site-id"})
    new_state: Optional[str] = field(default=None, metadata={FIELD_NAME: "new-state"})
    interface_name: Optional[str] = field(default=None, metadata={FIELD_NAME: "if-name"})
    vpn_id: Optional[str] = field(default=None, metadata={FIELD_NAME: "vpn-id"})
    viewed: Optional[bool] = field(default=None, metadata={FIELD_NAME: "acknowledged"})
    message: Optional[str] = field(default=None)
    values: Optional[list] = field(default=None)
    values_short_display: Optional[list] = field(default=None)
    event_name: Optional[str] = field(default=None, metadata={FIELD_NAME: "eventname"})
    rule_name: Optional[str] = field(default=None, metadata={FIELD_NAME: "rulename"})
    entry_time: Optional[int] = field(default=None)
    receive_time: Optional[int] = field(default=None)
    rule_name_display: Optional[str] = field(default=None)
    uuid: Optional[str] = field(default=None)
    possible_causes: Optional[List[str]] = field(default=None)
    consumed_events: Optional[list] = field(default=None)
    devices: Optional[list] = field(default=None)
    tenant: Optional[str] = field(default=None)
    id: Optional[str] = field(default=None)

    def issubset(self, other: "AlarmData") -> bool:
        field_keys = {field_key for field_key in self.__annotations__ if getattr(self, field_key)}
        return all(getattr(self, field_key) == getattr(other, field_key) for field_key in field_keys)

    def lowercase(self) -> "AlarmData":
        data = dict()
        for field_key in self.__annotations__:
            attr = getattr(self, field_key)
            try:
                data[field_key] = attr.lower()
            except AttributeError:
                data[field_key] = attr

        return AlarmData(**data)

    def alarm_severity_print(self) -> str:
        color = {
            Severity.CRITICAL: PrintColors.RED,
            Severity.MAJOR: PrintColors.YELLOW,
            Severity.MEDIUM: PrintColors.BLUE,
            Severity.MINOR: PrintColors.GREEN,
            Severity.UNKNOWN: PrintColors.NONE,
        }

        return f"{color[self.severity].value}{self.severity}{PrintColors.NONE.value}"

    def format_datetime(self, time: int) -> str:
        if time is None:
            return "N/A"
        return dt.datetime.fromtimestamp(time / 1e3).strftime("%H:%M:%S %Y-%m-%d")

    def __str__(self):
        result = (
            f"{self.__class__.__name__}: \n    "
            f"{self.message}\n    "
            f"{self.alarm_severity_print()}\n    "
            f"Alarm received at {self.format_datetime(self.receive_time)} "
            f"(entry time: {self.format_datetime(self.entry_time)}).\n    "
            f"Device {self.hostname} (system ip: {self.system_ip}).\n    "
            f"Alarm type: {self.name}."
        )
        return result


@define
class Device(DataclassBase):
    uuid: str
    personality: Personality = field(converter=Personality)
    id: str = field(metadata={FIELD_NAME: "deviceId"})
    hostname: str = field(metadata={FIELD_NAME: "host-name"})
    reachability: Reachability = field(converter=Reachability)
    local_system_ip: str = field(metadata={FIELD_NAME: "local-system-ip"})
    status: Optional[str] = field(default=None)
    memUsage: Optional[float] = field(default=None)
    mem_state: Optional[str] = field(default=None, metadata={FIELD_NAME: "memState"})
    cpu_state: Optional[str] = field(default=None, metadata={FIELD_NAME: "cpuState"})
    cpu_load: Optional[float] = field(default=None, metadata={FIELD_NAME: "cpuLoad"})
    state_description: Optional[str] = field(default=None)
    connected_vManages: List[str] = field(factory=list, metadata={FIELD_NAME: "connectedVManages"})
    model: Optional[str] = field(default=None, metadata={FIELD_NAME: "device-model"})
    board_serial: Optional[str] = field(default=None, metadata={FIELD_NAME: "board-serial"})
    vedgeCertificateState: Optional[str] = field(default=None, metadata={FIELD_NAME: "vedgeCertificateState"})  # TODO
    chasis_number: Optional[str] = field(default=None, metadata={FIELD_NAME: "chasisNumber"})
    site_id: Optional[str] = field(default=None, metadata={FIELD_NAME: "site-id"})
    site_name: Optional[str] = field(default=None, metadata={FIELD_NAME: "site-name"})

    @property
    def is_reachable(self) -> bool:
        return self.reachability is Reachability.REACHABLE


@define
class WanInterface(DataclassBase):
    color: str
    vDeviceIp: str = field(metadata={FIELD_NAME: "vdevice-name"})
    vDeviceName: str = field(metadata={FIELD_NAME: "vdevice-host-name"})
    adminState: str = field(metadata={FIELD_NAME: "admin-state"})
    interfaceName: str = field(metadata={FIELD_NAME: "interface"})
    privateIp: str = field(metadata={FIELD_NAME: "private-ip"})
    publicIp: str = field(metadata={FIELD_NAME: "public-ip"})
    privatePort: int = field(metadata={FIELD_NAME: "private-port"})
    publicPort: int = field(metadata={FIELD_NAME: "public-port"})
    operationalState: str = field(metadata={FIELD_NAME: "operation-state"})


@define
class Connection(DataclassBase):
    state: str
    peerType: str = field(metadata={FIELD_NAME: "peer-type"})
    systemIp: str = field(metadata={FIELD_NAME: "system-ip"})


@define
class BfdSessionData(DataclassBase):
    state: str
    siteId: str = field(metadata={FIELD_NAME: "site-id"})
    sourceTlocColor: str = field(metadata={FIELD_NAME: "local-color"})
    remoteTlocColor: str = field(metadata={FIELD_NAME: "color"})
    deviceIp: str = field(metadata={FIELD_NAME: "system-ip"})
    sourceIp: str = field(metadata={FIELD_NAME: "src-ip"})
    destinationPublicIp: str = field(metadata={FIELD_NAME: "dst-ip"})


@define
class OmpPeerData(DataclassBase):
    type: str
    state: str
    peerIp: str = field(metadata={FIELD_NAME: "peer"})
    siteId: str = field(metadata={FIELD_NAME: "site-id"})
    domainId: str = field(metadata={FIELD_NAME: "domain-id"})
    deviceIp: str = field(metadata={FIELD_NAME: "vdevice-name"})


@define
class OmpReceivedRouteData(DataclassBase):
    protocol: str
    peerIp: str = field(metadata={FIELD_NAME: "from-peer"})


@define
class OmpAdvertisedRouteData(DataclassBase):
    protocol: str
    peerIp: str = field(metadata={FIELD_NAME: "to-peer"})


@define
class OmpReceivedTlocData(DataclassBase):
    peerIp: str = field(metadata={FIELD_NAME: "from-peer"})


@define
class OmpAdvertisedTlocData(DataclassBase):
    peerIp: str = field(metadata={FIELD_NAME: "to-peer"})


@define
class OmpServiceData(DataclassBase):
    name: str = field(metadata={FIELD_NAME: "service"})
    status: Optional[str] = field(default=None)


@define
class OmpSummaryData(DataclassBase):
    oper_state: str = field(metadata={FIELD_NAME: "operstate"})
    admin_state: str = field(metadata={FIELD_NAME: "adminstate"})
    routes_received: str = field(metadata={FIELD_NAME: "routes-received"})
    tlocs_received: str = field(metadata={FIELD_NAME: "tlocs-received"})
    routes_sent: str = field(metadata={FIELD_NAME: "routes-sent"})
    tlocs_installed: str = field(metadata={FIELD_NAME: "tlocs-installed"})
    tlocs_sent: str = field(metadata={FIELD_NAME: "tlocs-sent"})
    vsmart_peers: str = field(metadata={FIELD_NAME: "vsmart-peers"})


@define
class EventData(DataclassBase):
    system_ip: str
    vmanage_system_ip: str
    tenant: str
    device_type: str
    component: str
    severity_level: str
    host_name: str
    event: str
    details: str
    event_name: str = field(metadata={FIELD_NAME: "eventname"})


@define(frozen=True)
class User(DataclassBase):
    username: str = field(metadata={FIELD_NAME: "userName"})
    password: Optional[str] = field(default=None)
    group: List[str] = field(factory=list)
    locale: Optional[str] = field(default=None)
    description: Optional[str] = field(default=None)
    resource_group: Optional[str] = field(default=None, metadata={FIELD_NAME: "resGroupName"})


@define(kw_only=True)
class TemplateInfo(DataclassBase):
    last_updated_by: str = field(metadata={FIELD_NAME: "lastUpdatedBy"})
    id: str = field(metadata={FIELD_NAME: "templateId"})
    factory_default: bool = field(metadata={FIELD_NAME: "factoryDefault"})
    name: str = field(metadata={FIELD_NAME: "templateName"})
    devices_attached: int = field(metadata={FIELD_NAME: "devicesAttached"})
    description: str = field(metadata={FIELD_NAME: "templateDescription"})
    last_updated_on: dt.datetime = field(metadata={FIELD_NAME: "lastUpdatedOn"})
    resource_group: Optional[str] = field(default=None, metadata={FIELD_NAME: "resourceGroup"})


@define(kw_only=True)
class FeatureTemplateInfo(TemplateInfo):
    template_type: str = field(metadata={FIELD_NAME: "templateType"})
    device_type: List[str] = field(metadata={FIELD_NAME: "deviceType"})
    version: str = field(metadata={FIELD_NAME: "templateMinVersion"})
    template_definiton: Optional[str] = field(default=None, metadata={FIELD_NAME: "templateDefinition"})


@define(kw_only=True)
class DeviceTemplateInfo(TemplateInfo):
    device_type: str = field(metadata={FIELD_NAME: "deviceType"})
    template_class: str = field(metadata={FIELD_NAME: "templateClass"})
    config_type: TemplateType = field(converter=TemplateType, metadata={FIELD_NAME: "configType"})
    template_attached: int = field(metadata={FIELD_NAME: "templateAttached"})
    draft_mode: Optional[str] = field(default=None, metadata={FIELD_NAME: "draftMode"})
    device_role: Optional[str] = field(default=None, metadata={FIELD_NAME: "deviceRole"})


@define
class Speedtest(DataclassBase):
    device_ip: str
    device_name: str
    destination_ip: str
    destination_name: str
    status: str
    up_speed: float
    down_speed: float


@define(frozen=True)
class PacketSetup(DataclassBase):
    session_id: str = field(metadata={FIELD_NAME: "sessionId"})
    is_new_session: bool = field(metadata={FIELD_NAME: "isNewSession"})


@define(frozen=True)
class Status(DataclassBase):
    file_download_status: Optional[str] = field(default=None, metadata={FIELD_NAME: "fileDownloadStatus"})
    file_size: Optional[int] = field(default=None, metadata={FIELD_NAME: "fileSize"})


@define(frozen=True)
class ServiceConfigurationData(DataclassBase):
    """Administration -> Service Configuration"""

    vmanage_id: str = field(metadata={FIELD_NAME: "vmanageID"})
    device_ip: str = field(metadata={FIELD_NAME: "deviceIP"})  # consider using ip4 module to verify
    services: dict = field(metadata={FIELD_NAME: "services"})  # consider using nested dataclasses
    persona: str = field(default="COMPUTE_AND_DATA")  # TODO Enum
    username: Optional[str] = field(default=None)
    password: Optional[str] = field(default=None)


@define(frozen=True)
class CloudConnectorData(DataclassBase):
    """Administration -> Settings -> SD-AVC Cloud Connector"""

    client_id: str = field(metadata={FIELD_NAME: "clientId"})
    client_secret: str = field(metadata={FIELD_NAME: "clientSecret"})
    org_name: str = field(metadata={FIELD_NAME: "orgName"})
    telemetry_enabled: bool = field(metadata={FIELD_NAME: "telemetryEnabled"})
    affinity: Optional[str] = field(default=None)
    cloud_enabled: bool = field(default=True, metadata={FIELD_NAME: "cloudEnabled"})


@define(frozen=True)
class CloudServicesSettings(DataclassBase):
    """Administration -> Settings -> Cloud Services"""

    enabled: bool = field(metadata={FIELD_NAME: "enabled"})
    otp: Optional[str] = field(default=None, metadata={FIELD_NAME: "otp"})
    cloud_gateway_url: Optional[str] = field(default=None, metadata={FIELD_NAME: "cloudGatewayUrl"})


@define(frozen=True)
class CloudOnRampForSaasMode(DataclassBase):
    """
    Administration -> Settings -> Cloud on Ramp for Saas
    """

    mode: str = field(metadata={FIELD_NAME: "mode"})


@define(frozen=True)
class TLOC:
    color: str
    encapsulation: str


@define(frozen=True)
class TierInfo(DataclassBase):
    """Endpoint: /dataservice/tier

    Since vManage 20.12 version, object has been renamed to "Resource Profile".
    """

    name: str = field(metadata={FIELD_NAME: "tierName"})
    vpn: int
    rid: int = field(metadata={FIELD_NAME: "@rid"})
    ipv4_route_limit_type: Optional[str] = field(default=None, metadata={FIELD_NAME: "ipv4RouteLimitType"})
    ipv4_route_limit_threshold: Optional[int] = field(default=None, metadata={FIELD_NAME: "ipv4RouteLimitThreshold"})
    ipv4_route_limit: Optional[int] = field(default=None, metadata={FIELD_NAME: "ipv4RouteLimit"})
    ipv6_route_limit_type: Optional[str] = field(default=None, metadata={FIELD_NAME: "ipv6RouteLimitType"})
    ipv6_route_limit_threshold: Optional[int] = field(default=None, metadata={FIELD_NAME: "ipv6RouteLimitThreshold"})
    ipv6_route_limit: Optional[int] = field(default=None, metadata={FIELD_NAME: "ipv6RouteLimit"})
    tlocs: List[TLOC] = field(default=[])
    # New in 20.12 version
    nat_session_limit: Optional[int] = field(default=None, metadata={FIELD_NAME: "natSessionLimit"})


@define(frozen=True)
class FeatureTemplateInformation(DataclassBase):
    """Endpoint: /dataservice/template/feature"""

    id: str = field(metadata={FIELD_NAME: "templateId"})
    name: str = field(metadata={FIELD_NAME: "templateName"})
    description: str = field(metadata={FIELD_NAME: "templateDescription"})
    type: str = field(metadata={FIELD_NAME: "templateType"})  # TODO Enum
    device_types: List[str] = field(metadata={FIELD_NAME: "deviceType"})  # TODO Enum
    last_updated_by: str = field(metadata={FIELD_NAME: "lastUpdatedBy"})
    last_updated_on: dt.datetime = field(metadata={FIELD_NAME: "lastUpdatedOn"})
    factory_default: bool = field(metadata={FIELD_NAME: "factoryDefault"})
    devices_attached: int = field(metadata={FIELD_NAME: "devicesAttached"})
    attached_masters: int = field(metadata={FIELD_NAME: "attachedMastersCount"})
    version: str = field(metadata={FIELD_NAME: "templateMinVersion"})
    config_type: str = field(metadata={FIELD_NAME: "configType"})
    created_by: str = field(metadata={FIELD_NAME: "createdBy"})
    created_on: dt.datetime = field(metadata={FIELD_NAME: "createdOn"})
    resource_group: str = field(metadata={FIELD_NAME: "resourceGroup"})


@define
class Organization(DataclassBase):
    name: str = field(metadata={FIELD_NAME: "org"})
    domain_id: int = field(metadata={FIELD_NAME: "domain-id"})
    control_connection_up: Optional[bool] = field(default=None, metadata={FIELD_NAME: "controlConnectionUp"})


@define
class Password(DataclassBase):
    old_password: str = field(metadata={FIELD_NAME: "oldpassword"})
    new_password: str = field(metadata={FIELD_NAME: "newpassword"})


@define
class Certificate(DataclassBase):
    controller_certificate: str = field(metadata={FIELD_NAME: "certificateSigning"})
    first_name: str = field(metadata={FIELD_NAME: "firstName"})
    last_name: str = field(metadata={FIELD_NAME: "lastName"})
    email: str = field(metadata={FIELD_NAME: "email"})
    validity_period: ValidityPeriod = field(metadata={FIELD_NAME: "validityPeriod"})
    retrieve_interval: int = field(converter=str, metadata={FIELD_NAME: "retrieveInterval"})

    @retrieve_interval.validator  # type: ignore
    def retrieve_interval_is_valid(self, attribute, value):
        RETRIEVE_INTERVAL_MAX = 60
        RETRIEVE_INTERVAL_MIN = 1
        if not RETRIEVE_INTERVAL_MIN <= int(value) <= RETRIEVE_INTERVAL_MAX:
            raise RetrieveIntervalOutOfRange("Retrieve interval must be value between 1 and 60 minutes")


@define
class Vbond(DataclassBase):
    vbond_address: str = field(metadata={FIELD_NAME: "domainIp"})
    vbond_port: str = field(metadata={FIELD_NAME: "port"})


@define(frozen=True)
class TenantAAA(DataclassBase):
    """
    Provider-Tenant -> Tenant -> Administration -> Manage users -> Remote AAA
    """

    accounting: bool = field(metadata={FIELD_NAME: "accounting"})
    admin_auth_order: bool = field(metadata={FIELD_NAME: "adminAuthOrder"})
    audit_disable: bool = field(metadata={FIELD_NAME: "auditDisable"})
    auth_fallback: bool = field(metadata={FIELD_NAME: "authFallback"})
    auth_order: List[str] = field(metadata={FIELD_NAME: "authOrder"})


@define(frozen=True)
class RadiusServer(DataclassBase):
    """
    Provider-Tenant -> Tenant -> Administration -> Manage users -> Remote AAA -> RADIUS
    """

    address: str = field(metadata={FIELD_NAME: "address"})
    auth_port: int = field(metadata={FIELD_NAME: "authPort"})
    acct_port: int = field(metadata={FIELD_NAME: "acctPort"})
    vpn: int = field(metadata={FIELD_NAME: "vpn"})
    vpn_ip_subnet: str = field(metadata={FIELD_NAME: "vpnIpSubnet"})
    key: str = field(metadata={FIELD_NAME: "key"})
    secret_key: str = field(metadata={FIELD_NAME: "secretKey"})
    priority: int = field(metadata={FIELD_NAME: "priority"})


@define(frozen=True)
class TenantRadiusServer(DataclassBase):
    """
    Provider-Tenant -> Tenant -> Administration -> Manage users -> Remote AAA -> RADIUS
    """

    timeout: int = field(default=3, metadata={FIELD_NAME: "timeout"})
    retransmit: int = field(default=5, metadata={FIELD_NAME: "retransmit"})
    servers: List[RadiusServer] = field(factory=list, metadata={FIELD_NAME: "server"})


@define(frozen=True)
class TacacsServer(DataclassBase):
    """
    Provider-Tenant -> Tenant -> Administration -> Manage users -> Remote AAA -> TACACS server
    """

    address: str = field(metadata={FIELD_NAME: "address"})
    auth_port: int = field(metadata={FIELD_NAME: "authPort"})
    vpn: int = field(metadata={FIELD_NAME: "vpn"})
    vpn_ip_subnet: str = field(metadata={FIELD_NAME: "vpnIpSubnet"})
    key: str = field(metadata={FIELD_NAME: "key"})
    secret_key: str = field(metadata={FIELD_NAME: "secretKey"})
    priority: int = field(metadata={FIELD_NAME: "priority"})


@define(frozen=True)
class TenantTacacsServer(DataclassBase):
    """
    Provider-Tenant -> Tenant -> Administration -> Manage users -> Remote AAA -> TACACS server
    """

    timeout: int = field(default=3, metadata={FIELD_NAME: "timeout"})
    authentication: str = field(default="PAP", metadata={FIELD_NAME: "authentication"})
    servers: List[TacacsServer] = field(factory=list, metadata={FIELD_NAME: "server"})


@define
class SoftwareInstallTimeout(DataclassBase):
    download_timeout_min: int = field(converter=str, metadata={FIELD_NAME: "downloadTimeoutInMin"})
    activate_timeout_min: int = field(converter=str, metadata={FIELD_NAME: "activateTimeoutInMin"})


class FeatureTemplatesTypes(BaseModel):
    model_config = ConfigDict(populate_by_name=True)
    parent: str
    default: str
    display_name: str = Field(serialization_alias="displayName", validation_alias="displayName")
    name: str
    type_class: str = Field(serialization_alias="typeClass", validation_alias="typeClass")
    description: str
    write_permission: bool
    read_permission: bool
    helper_type: List[str] = Field(default=[], serialization_alias="helperType", validation_alias="helperType")
    device_models: List[dict] = Field(default=[], serialization_alias="deviceModels", validation_alias="deviceModels")


@define
class ResourcePoolData(DataclassBase):
    """Endpoint: /resourcepool/resource/vpn"""

    tenant_id: str = field(metadata={FIELD_NAME: "tenantId"})
    tenant_vpn: int = field(metadata={FIELD_NAME: "tenantVpn"})
    device_vpn: Optional[int] = field(default=None, metadata={FIELD_NAME: "deviceVpn"})



# Copyright 2024 Cisco Systems, Inc. and its affiliates

from __future__ import annotations

import logging
from typing import TYPE_CHECKING, List

from catalystwan.endpoints.monitoring.status import MonitoringStatus, Status, UpdateStatus
from catalystwan.typed_list import DataSequence

logger = logging.getLogger(__name__)

if TYPE_CHECKING:
    from catalystwan.session import ManagerSession


class MonitoringStatusAPI:
    def __init__(self, session: ManagerSession) -> None:
        self.session = session
        self._endpoints = MonitoringStatus(session)

    def __str__(self) -> str:
        return str(self.session)

    def get_statistic_settings(self) -> DataSequence[Status]:
        """Get list of enabled statistics indexes

        Returns:
            DataSequence[EnabledIndex]: DataSequence of EnabledIndex objects
        """
        return self._endpoints.get_statistics_settings()

    def update_statistics_settings(self, payload: List[UpdateStatus]) -> DataSequence[Status]:
        """Update statistics indexes

        Args:
            payload (list[Status]): list of UpdateStatus objects

        Returns:
            DataSequence[Status]: DataSequence of Status objects
        """
        return self._endpoints.update_statistics_settings(payload)

    def get_disabled_devices_by_index(self, index_name: str) -> List[str]:
        """Get list of disabled devices for a statistics index

        Args:
            index_name (str): name of the statistics index

        Returns:
            List[str]: List of disables devices ips
        """
        return self._endpoints.get_disabled_device_list(index_name).root

    def update_disabled_devices_by_index(self, index_name: str, payload: List[str]) -> bool:
        """Update list of disabled devices for a statistics index

        Args:F
            index_name (str): name of the statistics index
            payload (list[str]): list of str objects

        Returns:
            bool: True if successful
        """
        return self._endpoints.update_statistics_device_list(index_name, payload).response

    def get_enabled_index_for_device(self, device_ip: str) -> List[str]:
        """Get list of enabled device for statistics index

        Args:
            device_ip (str): IP address of the device

        Returns:
            List[str]: List of enables indexes
        """
        return self._endpoints.get_enabled_index_for_device(device_ip).root



# Copyright 2024 Cisco Systems, Inc. and its affiliates

from typing import List

from pydantic import AliasPath, BaseModel, ConfigDict, Field, field_validator

from catalystwan.api.configuration_groups.parcel import Global, _ParcelBase, as_global
from catalystwan.models.policy.lists_entries import PolicerExceedAction


class PolicierEntry(BaseModel):
    model_config = ConfigDict(populate_by_name=True)
    burst: Global[int]
    exceed: Global[PolicerExceedAction]
    rate: Global[int]

    @field_validator("burst")
    @classmethod
    def check_burst(cls, burst_str: Global):
        assert 15000 <= burst_str.value <= 10_000_000
        return burst_str

    @field_validator("rate")
    @classmethod
    def check_rate(cls, rate_str: Global):
        assert 8 <= rate_str.value <= 100_000_000_000
        return rate_str


class PolicierParcel(_ParcelBase):
    entries: List[PolicierEntry] = Field(default=[], validation_alias=AliasPath("data", "entries"))

    def add_entry(self, burst: int, exceed: PolicerExceedAction, rate: int):
        self.entries.append(
            PolicierEntry(
                burst=as_global(burst),
                exceed=as_global(exceed, PolicerExceedAction),
                rate=as_global(rate),
            )
        )



from catalystwan.api.templates.device_variable import DeviceVariable
from catalystwan.utils.feature_template.find_template_values import find_template_values


def test_find_template_values():
    input_values = {
        "vpn-id": {"vipObjectType": "object", "vipType": "constant", "vipValue": 0},
        "name": {
            "vipObjectType": "object",
            "vipType": "ignore",
            "vipVariableName": "vpn_name",
        },
        "ecmp-hash-key": {
            "layer4": {
                "vipObjectType": "object",
                "vipType": "ignore",
                "vipValue": "false",
                "vipVariableName": "vpn_layer4",
            }
        },
        "nat64-global": {"prefix": {"stateful": {}}},
        "nat64": {
            "v4": {
                "pool": {
                    "vipType": "ignore",
                    "vipValue": [],
                    "vipObjectType": "tree",
                    "vipPrimaryKey": ["name"],
                }
            }
        },
        "nat": {
            "natpool": {
                "vipType": "ignore",
                "vipValue": [],
                "vipObjectType": "tree",
                "vipPrimaryKey": ["name"],
            },
            "port-forward": {
                "vipType": "ignore",
                "vipValue": [],
                "vipObjectType": "tree",
                "vipPrimaryKey": ["source-port", "translate-port"],
            },
            "static": {
                "vipType": "ignore",
                "vipValue": [],
                "vipObjectType": "tree",
                "vipPrimaryKey": ["source-ip", "translate-ip"],
            },
        },
        "route-import": {
            "vipType": "ignore",
            "vipValue": [],
            "vipObjectType": "tree",
            "vipPrimaryKey": ["protocol"],
        },
        "route-export": {
            "vipType": "ignore",
            "vipValue": [],
            "vipObjectType": "tree",
            "vipPrimaryKey": ["protocol"],
        },
        "dns": {
            "vipType": "constant",
            "vipValue": [
                {
                    "role": {
                        "vipType": "constant",
                        "vipValue": "primary",
                        "vipObjectType": "object",
                    },
                    "dns-addr": {
                        "vipType": "variableName",
                        "vipValue": "",
                        "vipObjectType": "object",
                        "vipVariableName": "vpn_dns_primary",
                    },
                    "priority-order": ["dns-addr", "role"],
                },
                {
                    "role": {
                        "vipType": "constant",
                        "vipValue": "secondary",
                        "vipObjectType": "object",
                    },
                    "dns-addr": {
                        "vipType": "variableName",
                        "vipValue": "",
                        "vipObjectType": "object",
                        "vipVariableName": "vpn_dns_secondary",
                    },
                    "priority-order": ["dns-addr", "role"],
                },
            ],
            "vipObjectType": "tree",
            "vipPrimaryKey": ["dns-addr"],
        },
        "host": {
            "vipType": "ignore",
            "vipValue": [],
            "vipObjectType": "tree",
            "vipPrimaryKey": ["hostname"],
        },
        "service": {
            "vipType": "ignore",
            "vipValue": [],
            "vipObjectType": "tree",
            "vipPrimaryKey": ["svc-type"],
        },
        "ip": {
            "route": {
                "vipType": "constant",
                "vipValue": [
                    {
                        "prefix": {
                            "vipObjectType": "object",
                            "vipType": "constant",
                            "vipValue": "0.0.0.0/0",
                            "vipVariableName": "vpn_ipv4_ip_prefix",
                        },
                        "next-hop": {
                            "vipType": "constant",
                            "vipValue": [
                                {
                                    "address": {
                                        "vipObjectType": "object",
                                        "vipType": "variableName",
                                        "vipValue": "",
                                        "vipVariableName": "vpn_next_hop_ip_address_0",
                                    },
                                    "distance": {
                                        "vipObjectType": "object",
                                        "vipType": "ignore",
                                        "vipValue": 1,
                                        "vipVariableName": "vpn_next_hop_ip_distance_0",
                                    },
                                    "priority-order": ["address", "distance"],
                                }
                            ],
                            "vipObjectType": "tree",
                            "vipPrimaryKey": ["address"],
                        },
                        "priority-order": ["prefix", "next-hop", "next-hop-with-track"],
                    }
                ],
                "vipObjectType": "tree",
                "vipPrimaryKey": ["prefix"],
            },
            "gre-route": {},
            "ipsec-route": {},
            "service-route": {},
        },
        "ipv6": {},
        "omp": {
            "advertise": {
                "vipType": "ignore",
                "vipValue": [],
                "vipObjectType": "tree",
                "vipPrimaryKey": ["protocol"],
            },
            "ipv6-advertise": {
                "vipType": "ignore",
                "vipValue": [],
                "vipObjectType": "tree",
                "vipPrimaryKey": ["protocol"],
            },
        },
    }
    expected_values = {
        "vpn-id": 0,
        "dns": [
            {"role": "primary", "dns-addr": DeviceVariable(name="vpn_dns_primary")},
            {"role": "secondary", "dns-addr": DeviceVariable(name="vpn_dns_secondary")},
        ],
        "ip": {
            "route": [
                {
                    "prefix": "0.0.0.0/0",
                    "next-hop": [
                        {
                            "address": DeviceVariable(name="vpn_next_hop_ip_address_0"),
                        }
                    ],
                }
            ],
        },
    }
    # Act
    result = find_template_values(input_values)
    # Assert
    assert expected_values == result



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from pathlib import Path
from typing import ClassVar, Optional

from pydantic import ConfigDict, Field

from catalystwan.api.templates.feature_template import FeatureTemplate


class CiscoBannerModel(FeatureTemplate):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    login_banner: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "login"})
    motd_banner: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "motd"})

    payload_path: ClassVar[Path] = Path(__file__).parent / "DEPRECATED"
    type: ClassVar[str] = "cisco_banner"



# Copyright 2024 Cisco Systems, Inc. and its affiliates

from typing import List, Literal, Optional, Union
from uuid import UUID

from pydantic import BaseModel, ConfigDict, Field

from catalystwan.api.configuration_groups.parcel import Default, Global, Variable
from catalystwan.models.configuration.feature_profile.common import Prefix

EigrpAuthType = Literal[
    "md5",
    "hmac-sha-256",
]


RedistributeProtocol = Literal[
    "bgp",
    "connected",
    "nat-route",
    "omp",
    "ospf",
    "ospfv3",
    "static",
]


class KeychainDetails(BaseModel):
    key_id: Union[Global[int], Variable, Default[None]] = Field(serialization_alias="keyId", validation_alias="keyId")
    keystring: Union[Global[str], Variable, Default[None]]


class EigrpAuthentication(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    auth_type: Union[Global[EigrpAuthType], Variable, Default[None]] = Field(
        serialization_alias="type", validation_alias="type"
    )
    auth_key: Optional[Union[Global[str], Variable, Default[None]]] = Field(
        serialization_alias="authKey", validation_alias="authKey"
    )
    key: Optional[List[KeychainDetails]] = Field(serialization_alias="key", validation_alias="key")


class TableMap(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    name: Optional[Union[Default[None], Global[UUID]]] = Default[None](value=None)
    filter: Optional[Union[Global[bool], Variable, Default[bool]]] = Default[bool](value=False)


class SummaryAddress(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    prefix: Prefix


class IPv4StaticRoute(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    name: Union[Global[str], Variable]
    shutdown: Optional[Union[Global[int], Variable, Default[bool]]] = Default[bool](value=False)
    summary_address: Optional[List[SummaryAddress]] = Field(
        serialization_alias="summaryAddress", validation_alias="summaryAddress"
    )


class RedistributeIntoEigrp(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    protocol: Union[Global[RedistributeProtocol], Variable]
    route_policy: Optional[Union[Default[None], Global[UUID]]] = Default[None](value=None)


class AddressFamily(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    redistribute: Optional[List[RedistributeIntoEigrp]] = None
    network: List[SummaryAddress]


class EigrpData(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    as_number: Union[Global[int], Variable] = Field(serialization_alias="asNum", validation_alias="asNum")
    address_family: AddressFamily = Field(serialization_alias="addressFamily", validation_alias="addressFamily")
    hello_interval: Union[Global[int], Variable, Default[int]] = Field(
        serialization_alias="helloInterval", validation_alias="helloInterval", default=Default[int](value=5)
    )
    hold_time: Union[Global[int], Variable, Default[int]] = Field(
        serialization_alias="holdTime", validation_alias="holdTime", default=Default[int](value=15)
    )
    authentication: Optional[EigrpAuthentication] = None
    af_interface: Optional[List[IPv4StaticRoute]] = Field(
        serialization_alias="afInterface", validation_alias="afInterface", default=None
    )
    table_map: TableMap = Field(serialization_alias="tableMap", validation_alias="tableMap", default=TableMap())


class EigrpCreationPayload(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    name: str
    description: Optional[str] = None
    data: EigrpData
    metadata: Optional[dict] = None



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field, field_validator

from catalystwan.api.templates.device_variable import DeviceVariable
from catalystwan.api.templates.feature_template import FeatureTemplate
from catalystwan.utils.dict import merge
from catalystwan.utils.pydantic_field import get_extra_field


class FeatureTemplateOptionType(str, Enum):
    CONSTANT = "constant"
    VARIABLE = "variable"
    IGNORE = "ignore"
    NOT_IGNORE = "notIgnore"
    VARIABLE_NAME = "variableName"


class FeatureTemplateObjectType(str, Enum):
    OBJECT = "object"
    TREE = "tree"
    LIST = "list"  # TODO Use comma(,) for multiple values
    NODE_ONLY = "node-only"  # No information about the value


class VipVariable(BaseModel):
    # class Config:
    #     arbitrary_types_allowed = True
    #     allow_population_by_field_name = True

    value: Any = Field(alias="vipValue")
    type: FeatureTemplateOptionType = Field(alias="vipType")
    object_type: FeatureTemplateObjectType = Field(alias="vipObjectType")
    device_specific_name: Optional[str] = Field(default=None, alias="vipVariableName")
    primary_key: Optional[Any] = Field(default=None, alias="vipPrimaryKey")


def get_path_dict(paths: List[List[str]]) -> dict:
    """Builds a tree like structure out of a list of paths"""

    def _recurse(tmp_dict: dict, chain: List[str]):
        if len(chain) == 0:
            return

        key, *tail = chain

        if key not in tmp_dict:
            tmp_dict[key] = dict()  # Value for every new dict. Changed in-place.

        _recurse(tmp_dict[key], tail)
        return

    new_path_dict: Dict[str, Any] = {}
    for path in paths:
        _recurse(new_path_dict, path)

    return new_path_dict


class FeatureTemplateField(BaseModel):
    # TODO aliases
    key: str
    description: str = ""
    details: str = ""
    optionType: List[FeatureTemplateOptionType]
    defaultOption: FeatureTemplateOptionType
    dataPath: List[str] = []
    objectType: FeatureTemplateObjectType
    dataType: Dict[str, Any] = {}
    primaryKeys: List[str] = []
    children: List[FeatureTemplateField] = []

    @field_validator("dataType", mode="before")
    @classmethod
    def convert_data_type_to_dict(cls, value):
        if isinstance(value, str):
            return {"type": value}
        return value

    def data_path(self, output):
        for child in self.children:
            output.update(child.data_path(output))
        output.update(get_path_dict([t.dataPath for t in self.children]))

        return output

    # value must be JSON serializable, return JSON serializable dict
    def payload_scheme(
        self, value: Any = None, json_dumped_value: Any = None, priority_order=None, vip_type=None
    ) -> dict:
        output: dict = {}
        rel_output: dict = {}
        rel_output.update(get_path_dict([self.dataPath]))

        output["vipObjectType"] = self.objectType.value

        def nest_value_in_output(value: Any) -> dict:
            pointer = rel_output
            for path in self.dataPath:
                pointer = pointer[path]
            pointer[self.key] = value
            return rel_output

        if isinstance(value, DeviceVariable):
            vip_variable = VipVariable(
                vipValue="",
                vipType=FeatureTemplateOptionType.VARIABLE_NAME,
                vipObjectType=self.objectType,
                vipVariableName=value.name,
            )
            return nest_value_in_output(vip_variable.model_dump(by_alias=True, exclude_none=True))

        if value is not None and not self.children:
            output["vipType"] = vip_type or FeatureTemplateOptionType.CONSTANT.value
            output["vipValue"] = json_dumped_value
        elif value is not None and self.children:
            output["vipType"] = vip_type or FeatureTemplateOptionType.CONSTANT.value
            children_output = []
            for obj in value:
                obj_json_dump = obj.model_dump(mode="json")
                child_payload: dict = {}
                for child in self.children:  # Child in schema
                    obj: FeatureTemplate  # type: ignore

                    # We are searching for the model field that
                    # corresponds to the child element (field from vManage schema)
                    # If this line fails, it means that the schema from vManage has more fields than the model
                    # and the model needs to be updated (add missing fields)
                    model_tuple = next(
                        filter(
                            lambda f: (
                                f[1].alias == child.key
                                or get_extra_field(f[1], "vmanage_key") == child.key
                                or f[0] == child.key
                            ),
                            obj.model_fields.items(),
                        )
                    )
                    # We get the pydantic model field and the value from the object
                    model_field = model_tuple[1]
                    obj_value = getattr(obj, model_tuple[0])
                    obj_json_value = obj_json_dump.get(model_tuple[0])
                    po = get_extra_field(model_field, "priority_order")
                    vip_type = get_extra_field(model_field, "vip_type")
                    # After we get the data, we create the payload by populating the schema
                    # Then we merge the payload to the output
                    merge(
                        child_payload,
                        child.payload_scheme(
                            obj_value,
                            json_dumped_value=obj_json_value,
                            priority_order=po,
                            vip_type=vip_type,
                        ),
                    )
                    if priority_order:
                        child_payload.update({"priority-order": priority_order})
                children_output.append(child_payload)
            output["vipValue"] = children_output
        else:
            if value is None or "default" in self.dataType:
                return {}
            else:
                output["vipValue"] = []
                output["vipType"] = FeatureTemplateOptionType.IGNORE.value

        if self.primaryKeys:
            output["vipPrimaryKey"] = self.primaryKeys

        return nest_value_in_output(output)



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from typing import List, Literal
from uuid import UUID

from pydantic import BaseModel, ConfigDict, Field

from catalystwan.models.policy.policy_definition import (
    DefinitionWithSequencesCommonBase,
    PLPEntryType,
    PolicyDefinitionBase,
)


class RewritePolicyHeader(PolicyDefinitionBase):
    type: Literal["rewriteRule"] = "rewriteRule"


class RewritePolicyRule(BaseModel):
    class_: UUID = Field(serialization_alias="class", validation_alias="class")
    plp: str
    dscp: str
    l2cos: str = Field(serialization_alias="layer2Cos", validation_alias="layer2Cos")
    model_config = ConfigDict(populate_by_name=True)


class RewritePolicyDefinition(BaseModel):
    rules: List[RewritePolicyRule] = []


class RewritePolicy(RewritePolicyHeader, DefinitionWithSequencesCommonBase):
    definition: RewritePolicyDefinition = RewritePolicyDefinition()

    def add_rule(self, class_map_ref: UUID, dscp: int, l2cos: int, plp: PLPEntryType) -> None:
        self.definition.rules.append(RewritePolicyRule(class_=class_map_ref, plp=plp, dscp=str(dscp), l2cos=str(l2cos)))

    model_config = ConfigDict(populate_by_name=True)



# Copyright 2024 Cisco Systems, Inc. and its affiliates

from typing import List

from pydantic import BaseModel, Field

from catalystwan.models.policy import (
    AnyPolicyDefinition,
    AnyPolicyList,
    CentralizedPolicy,
    LocalizedPolicy,
    SecurityPolicy,
)


class UX1Policies(BaseModel):
    centralized_policies: List[CentralizedPolicy] = Field(default=[], serialization_alias="centralizedPolicies")
    localized_policies: List[LocalizedPolicy] = Field(default=[], serialization_alias="localizedPolicies")
    security_policies: List[SecurityPolicy] = Field(default=[], serialization_alias="securityPolicies")
    policy_definitions: List[AnyPolicyDefinition] = Field(default=[], serialization_alias="policyDefinitions")
    policy_lists: List[AnyPolicyList] = Field(default=[], serialization_alias="policyLists")


class UX1Templates(BaseModel):
    pass


class UX1Config(BaseModel):
    # All UX1 Configuration items - Mega Model
    policies: UX1Policies
    templates: UX1Templates


class UX2Config(BaseModel):
    # All UX2 Configuration items - Mega Model
    pass



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from enum import Enum
from pathlib import Path
from typing import ClassVar, List, Optional

from pydantic import ConfigDict, Field

from catalystwan.api.templates.bool_str import BoolStr
from catalystwan.api.templates.feature_template import FeatureTemplate, FeatureTemplateValidator

DEFAULT_OMP_HOLDTIME = 60
DEFAULT_OMP_EOR_TIMER = 300
DEFAULT_OMP_GRACEFUL_RESTART_TIMER = 43200
DEFAULT_OMP_ADVERTISEMENT_INTERVAL = 1
DEFAULT_OMP_SENDPATH_LIMIT = 4
DEFAULT_OMP_ECMP_LIMIT = 4


class IPv4AdvertiseProtocol(str, Enum):
    BGP = "bgp"
    OSPF = "ospf"
    OSPFV3 = "ospfv3"
    CONNECTED = "connected"
    STATIC = "static"
    EIGRP = "eigrp"
    LISP = "lisp"
    ISIS = "isis"


class Route(str, Enum):
    EXTERNAL = "external"


class IPv4Advertise(FeatureTemplateValidator):
    protocol: IPv4AdvertiseProtocol
    route: Optional[Route] = None


class IPv6AdvertiseProtocol(str, Enum):
    BGP = "bgp"
    OSPF = "ospf"
    CONNECTED = "connected"
    STATIC = "static"
    EIGRP = "eigrp"
    LISP = "lisp"
    ISIS = "isis"


class IPv6Advertise(FeatureTemplateValidator):
    protocol: IPv6AdvertiseProtocol


class TransportGateway(str, Enum):
    PREFER = "prefer"
    ECMP_WITH_DIRECT_PATH = "ecmp-with-direct-path"


class SiteTypes(str, Enum):
    TYPE_1 = "type-1"
    TYPE_2 = "type-2"
    TYPE_3 = "type-3"
    CLOUD = "cloud"
    BRANCH = "branch"
    BR = "br"
    SPOKE = "spoke"


class CiscoOMPModel(FeatureTemplate):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    graceful_restart: Optional[BoolStr] = Field(default=True, json_schema_extra={"vmanage_key": "graceful-restart"})
    overlay_as: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "overlay-as"})
    send_path_limit: Optional[int] = Field(
        DEFAULT_OMP_SENDPATH_LIMIT, json_schema_extra={"vmanage_key": "send-path-limit"}
    )
    ecmp_limit: Optional[int] = Field(DEFAULT_OMP_ECMP_LIMIT, json_schema_extra={"vmanage_key": "ecmp-limit"})
    shutdown: Optional[BoolStr] = None
    omp_admin_distance_ipv4: Optional[int] = Field(
        default=None, json_schema_extra={"vmanage_key": "omp-admin-distance-ipv4"}
    )
    omp_admin_distance_ipv6: Optional[int] = Field(
        default=None, json_schema_extra={"vmanage_key": "omp-admin-distance-ipv6"}
    )
    advertisement_interval: Optional[int] = Field(
        DEFAULT_OMP_ADVERTISEMENT_INTERVAL,
        json_schema_extra={"vmanage_key": "advertisement-interval", "data_path": ["timers"]},
    )
    graceful_restart_timer: Optional[int] = Field(
        DEFAULT_OMP_GRACEFUL_RESTART_TIMER,
        json_schema_extra={"vmanage_key": "graceful-restart-timer", "data_path": ["timers"]},
    )
    eor_timer: Optional[int] = Field(
        DEFAULT_OMP_EOR_TIMER, json_schema_extra={"vmanage_key": "eor-timer", "data_path": ["timers"]}
    )
    holdtime: Optional[int] = Field(DEFAULT_OMP_HOLDTIME, json_schema_extra={"data_path": ["timers"]})
    advertise: Optional[List[IPv4Advertise]] = None
    ipv6_advertise: Optional[List[IPv6Advertise]] = Field(
        default=None, json_schema_extra={"vmanage_key": "ipv6-advertise"}
    )
    ignore_region_path_length: Optional[BoolStr] = Field(
        default=False, json_schema_extra={"vmanage_key": "ignore-region-path-length"}
    )
    transport_gateway: Optional[TransportGateway] = Field(
        default=None, json_schema_extra={"vmanage_key": "transport-gateway"}
    )
    site_types: Optional[List[SiteTypes]] = Field(default=None, json_schema_extra={"vmanage_key": "site-types"})
    auto_translate: Optional[BoolStr] = Field(default=False, json_schema_extra={"vmanage_key": "auto-translate"})

    payload_path: ClassVar[Path] = Path(__file__).parent / "DEPRECATED"
    type: ClassVar[str] = "cisco_omp"



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from enum import Enum
from pathlib import Path
from typing import ClassVar, List, Optional

from pydantic import ConfigDict, Field

from catalystwan.api.templates.bool_str import BoolStr
from catalystwan.api.templates.feature_template import FeatureTemplate, FeatureTemplateValidator


class Oid(FeatureTemplateValidator):
    id: str
    exclude: Optional[BoolStr] = None


class View(FeatureTemplateValidator):
    name: str
    oid: Optional[List[Oid]] = None


class Authorization(str, Enum):
    READ_ONLY = "read-only"


class Community(FeatureTemplateValidator):
    name: str
    view: str
    authorization: Authorization


class SecurityLevel(str, Enum):
    NOAUTHNOPRIV = "no-auth-no-priv"
    AUTHNOPRIV = "auth-no-priv"
    AUTHPRIV = "auth-priv"


class Group(FeatureTemplateValidator):
    name: str
    security_level: SecurityLevel = Field(json_schema_extra={"vmanage_key": "security-level"})
    view: str
    model_config = ConfigDict(populate_by_name=True)


class Auth(str, Enum):
    MD5 = "md5"
    SHA = "sha"


class Priv(str, Enum):
    AES_CFB_128 = "aes-cfb-128"


class User(FeatureTemplateValidator):
    name: str
    auth: Optional[Auth] = None
    auth_password: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "auth-password"})
    priv: Optional[Priv] = None
    priv_password: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "priv-password"})
    group: str
    model_config = ConfigDict(populate_by_name=True)


class Target(FeatureTemplateValidator):
    vpn_id: int = Field(json_schema_extra={"vmanage_key": "vpn-id"})
    ip: str
    port: int
    community_name: str = Field(default=None, json_schema_extra={"vmanage_key": "community-name"})
    user: Optional[str] = None
    source_interface: str = Field(default=None, json_schema_extra={"vmanage_key": "source-interface"})
    model_config = ConfigDict(populate_by_name=True)


class CiscoSNMPModel(FeatureTemplate):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    shutdown: Optional[BoolStr] = True
    contact: Optional[str] = None
    location: Optional[str] = None
    view: Optional[List[View]] = None
    community: Optional[List[Community]] = None
    group: Optional[List[Group]] = None
    user: Optional[List[User]] = None
    target: Optional[List[Target]] = Field(default=None, json_schema_extra={"data_path": ["trap"]})

    payload_path: ClassVar[Path] = Path(__file__).parent / "DEPRECATED"
    type: ClassVar[str] = "cisco_snmp"



import unittest
from unittest.mock import MagicMock, patch

from catalystwan.endpoints.url_monitoring import UrlMonitoring
from catalystwan.models.url_monitoring import DeleteUrlMonitorRequest, UrlMonitoringConfig


class TestUrlMonitoring(unittest.TestCase):
    @patch("catalystwan.session.ManagerSession")
    def setUp(self, session_mock) -> None:
        self.session = session_mock

        self.mock_url_monitor_info_data = {"url": "/client/server/ready", "threshold": 98, "alarm_raised": False}

        self.mock_url_monitor_config_add_data = {"url": "/client/server/ready", "threshold": 98}
        self.mock_url_monitor_config_update_data = {"url": "/client/server/ready", "threshold": 90}
        self.mock_url_monitor_delete_url = DeleteUrlMonitorRequest(url="/client/server/ready")

    @patch("catalystwan.session.ManagerSession")
    def test_add_url_monitor(self, mock_session):
        # Arrange
        mock_response = MagicMock()
        mock_session.get_data.return_value = UrlMonitoringConfig(**self.mock_url_monitor_config_add_data)
        # Mock the request method of the ManagerSession to return the mock response
        mock_session_instance = mock_session.return_value
        mock_session_instance.request.return_value = mock_response
        mock_session.post.return_value = mock_response
        url_monitor = UrlMonitoring(mock_session_instance)

        # Act
        response = url_monitor.add_url_monitor(self.mock_url_monitor_config_add_data)
        # Assert
        self.assertEqual(response, None)
        # Ensure the request method was called
        mock_session_instance.request.assert_called_once()

    @patch("catalystwan.session.ManagerSession")
    @patch("catalystwan.response.ManagerResponse")
    def test_get_url_monitor(self, mock_session, mock_response):
        # Arrange
        mock_response = MagicMock()
        mock_session_instance = mock_session.return_value
        mock_session_instance.request.return_value = mock_response
        url_monitor = UrlMonitoring(mock_session_instance)
        observed_url_monitor = url_monitor.get_url_monitor()
        # Assert
        self.assertIsNotNone(observed_url_monitor)

    @patch("catalystwan.session.ManagerSession")
    def test_update_url_monitor(self, mock_session):
        # Arrange
        mock_response = MagicMock()
        mock_session.get_data.return_value = UrlMonitoringConfig(**self.mock_url_monitor_config_update_data)
        # Mock the request method of the ManagerSession to return the mock response
        mock_session_instance = mock_session.return_value
        mock_session_instance.request.return_value = mock_response
        url_monitor = UrlMonitoring(mock_session_instance)
        # Act
        response = url_monitor.update_url_monitor(self.mock_url_monitor_config_update_data)
        # Assert
        self.assertEqual(response, None)
        # Ensure the request method was called
        mock_session_instance.request.assert_called_once()

    @patch("catalystwan.session.ManagerSession")
    def test_delete_url_monitor(self, mock_session):
        # Arrange
        mock_response = MagicMock()
        mock_session.get_data.return_value = self.mock_url_monitor_delete_url
        # Mock the request method of the ManagerSession to return the mock response
        mock_session_instance = mock_session.return_value
        mock_session_instance.request.return_value = mock_response
        url_monitor = UrlMonitoring(mock_session_instance)
        # Act
        response = url_monitor.delete_url_monitor(self.mock_url_monitor_delete_url)
        # Assert
        self.assertEqual(response, None)
        # Ensure the request method was called
        mock_session_instance.request.assert_called_once()
